if polyglot#init#is_disabled(expand('<sfile>:p'), 'xsd', 'indent/xsd.vim')
  finish
endif

" Vim indent file
" Language: 	.xsd files (XML Schema)
" Maintainer:	Nobody
" Last Change:	2005 Jun 09

" Only load this indent file when no other was loaded.
if exists("b:did_indent")
  finish
endif

" Use XML formatting rules
runtime! indent/xml.vim

