if polyglot#init#is_disabled(expand('<sfile>:p'), 'less', 'indent/less.vim')
  finish
endif

" Vim indent file
" Language:	less
" Maintainer:	Alessandro Vioni <jenoma@gmail.com>
" URL: https://github.com/genoma/vim-less
" Last Change:	2014 November 24

if exists("b:did_indent")
  finish
endif

runtime! indent/css.vim

" vim:set sw=2:
