if polyglot#init#is_disabled(expand('<sfile>:p'), 'typescript', 'syntax/yats/web-window.vim')
  finish
endif

syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName applicationCache
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName closed
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName Components
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName controllers
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName dialogArguments
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName document
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName frameElement
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName frames
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName fullScreen
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName history
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName innerHeight
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName innerWidth
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName length
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName location
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName locationbar
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName menubar
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName messageManager
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName name navigator
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName opener
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName outerHeight
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName outerWidth
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName pageXOffset
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName pageYOffset
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName parent
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName performance
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName personalbar
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName returnValue
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName screen
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName screenX
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName screenY
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName scrollbars
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName scrollMaxX
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName scrollMaxY
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName scrollX
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName scrollY
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName self sidebar
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName status
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName statusbar
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName toolbar
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName top visualViewport
syntax keyword typescriptBOMWindowProp containedin=typescriptIdentifierName window
syntax cluster props add=typescriptBOMWindowProp
if exists("did_typescript_hilink") | HiLink typescriptBOMWindowProp Structure
endif
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName alert nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName atob nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName blur nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName btoa nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName clearImmediate nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName clearInterval nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName clearTimeout nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName close nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName confirm nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName dispatchEvent nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName find nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName focus nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName getAttention nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName getAttentionWithCycleCount nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName getComputedStyle nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName getDefaulComputedStyle nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName getSelection nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName matchMedia nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName maximize nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName moveBy nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName moveTo nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName open nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName openDialog nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName postMessage nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName print nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName prompt nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName removeEventListener nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName resizeBy nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName resizeTo nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName restore nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName scroll nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName scrollBy nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName scrollByLines nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName scrollByPages nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName scrollTo nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName setCursor nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName setImmediate nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName setInterval nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName setResizable nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName setTimeout nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName showModalDialog nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName sizeToContent nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName stop nextgroup=typescriptFuncCallArg
syntax keyword typescriptBOMWindowMethod containedin=typescriptIdentifierName updateCommands nextgroup=typescriptFuncCallArg
syntax cluster props add=typescriptBOMWindowMethod
if exists("did_typescript_hilink") | HiLink typescriptBOMWindowMethod Structure
endif
syntax keyword typescriptBOMWindowEvent contained onabort onbeforeunload onblur onchange
syntax keyword typescriptBOMWindowEvent contained onclick onclose oncontextmenu ondevicelight
syntax keyword typescriptBOMWindowEvent contained ondevicemotion ondeviceorientation
syntax keyword typescriptBOMWindowEvent contained ondeviceproximity ondragdrop onerror
syntax keyword typescriptBOMWindowEvent contained onfocus onhashchange onkeydown onkeypress
syntax keyword typescriptBOMWindowEvent contained onkeyup onload onmousedown onmousemove
syntax keyword typescriptBOMWindowEvent contained onmouseout onmouseover onmouseup
syntax keyword typescriptBOMWindowEvent contained onmozbeforepaint onpaint onpopstate
syntax keyword typescriptBOMWindowEvent contained onreset onresize onscroll onselect
syntax keyword typescriptBOMWindowEvent contained onsubmit onunload onuserproximity
syntax keyword typescriptBOMWindowEvent contained onpageshow onpagehide
if exists("did_typescript_hilink") | HiLink typescriptBOMWindowEvent Keyword
endif
syntax keyword typescriptBOMWindowCons containedin=typescriptIdentifierName DOMParser
syntax keyword typescriptBOMWindowCons containedin=typescriptIdentifierName QueryInterface
syntax keyword typescriptBOMWindowCons containedin=typescriptIdentifierName XMLSerializer
if exists("did_typescript_hilink") | HiLink typescriptBOMWindowCons Structure
endif
