if polyglot#init#is_disabled(expand('<sfile>:p'), 'mf', 'indent/mf.vim')
  finish
endif

" METAFONT indent file
" Language:    METAFONT
" Maintainer:  Nicola Vitacolonna <nvitacolonna@gmail.com>
" Last Change: 2016 Oct 1

runtime! indent/mp.vim
