if polyglot#init#is_disabled(expand('<sfile>:p'), 'sbt', 'ftplugin/sbt.vim')
  finish
endif

" Vim filetype plugin file
" Language:     sbt
" Maintainer:   Steven Dobay <stevendobay at protonmail.com>
" License:      Same as Vim
" Last Change:  2017.04.30
" ----------------------------------------------------------------------------

if exists('b:did_ftplugin') || &cp
  finish
endif

let b:did_ftplugin = 1

runtime! ftplugin/scala.vim

