if polyglot#init#is_disabled(expand('<sfile>:p'), 'mathematica', 'after/syntax/mma.vim')
  finish
endif

"Vim conceal file
" Language: Mathematica
" Maintainer: R. Menon <rsmenon@icloud.com>
" Last Change: Feb 25, 2013

if (exists('g:mma_candy') && g:mma_candy == 0) || !has('conceal') || &enc != 'utf-8'
    finish
endif

"These are fairly safe and straightforward conceals
if exists('g:mma_candy') && g:mma_candy > 0
	"Rules
	syntax match mmaOperator "->" conceal cchar=→ "Rule
	syntax match mmaOperator ":>" conceal cchar=⧴ "RuleDelayed

	"Logicals
	syntax match mmaOperator "===" conceal cchar=≡ "SameQ
	syntax match mmaOperator "=!=" conceal cchar=≢ "UnsameQ
	syntax match mmaOperator "!="  conceal cchar=≠ "NotEqual
	syntax match mmaOperator "<="  conceal cchar=≤ "LessEqual
	syntax match mmaOperator ">="  conceal cchar=≥ "GreaterEqual

	"Constants
	syntax keyword mmaSystemSymbol Pi       conceal cchar=π
	syntax keyword mmaSystemSymbol Infinity conceal cchar=∞
	syntax keyword mmaSystemSymbol Degree   conceal cchar=°

	"Domains
	syntax keyword mmaSystemSymbol Reals     conceal cchar=ℝ
	syntax keyword mmaSystemSymbol Integers  conceal cchar=ℤ
	syntax keyword mmaSystemSymbol Complexes conceal cchar=ℂ
	syntax keyword mmaSystemSymbol Rationals conceal cchar=ℚ

	"Greek
    "Scratched from https://reference.wolfram.com/language/guide/ListingOfNamedCharacters.html
    syntax match mmaSymbol "\\\[FormalA\]"                         conceal cchar=a
    syntax match mmaSymbol "\\\[FormalB\]"                         conceal cchar=b
    syntax match mmaSymbol "\\\[FormalC\]"                         conceal cchar=c
    syntax match mmaSymbol "\\\[FormalD\]"                         conceal cchar=d
    syntax match mmaSymbol "\\\[FormalE\]"                         conceal cchar=e
    syntax match mmaSymbol "\\\[FormalF\]"                         conceal cchar=f
    syntax match mmaSymbol "\\\[FormalG\]"                         conceal cchar=g
    syntax match mmaSymbol "\\\[FormalH\]"                         conceal cchar=h
    syntax match mmaSymbol "\\\[FormalI\]"                         conceal cchar=i
    syntax match mmaSymbol "\\\[FormalJ\]"                         conceal cchar=j
    syntax match mmaSymbol "\\\[FormalK\]"                         conceal cchar=k
    syntax match mmaSymbol "\\\[FormalL\]"                         conceal cchar=l
    syntax match mmaSymbol "\\\[FormalM\]"                         conceal cchar=m
    syntax match mmaSymbol "\\\[FormalN\]"                         conceal cchar=n
    syntax match mmaSymbol "\\\[FormalO\]"                         conceal cchar=o
    syntax match mmaSymbol "\\\[FormalP\]"                         conceal cchar=p
    syntax match mmaSymbol "\\\[FormalQ\]"                         conceal cchar=q
    syntax match mmaSymbol "\\\[FormalR\]"                         conceal cchar=r
    syntax match mmaSymbol "\\\[FormalS\]"                         conceal cchar=s
    syntax match mmaSymbol "\\\[FormalT\]"                         conceal cchar=t
    syntax match mmaSymbol "\\\[FormalU\]"                         conceal cchar=u
    syntax match mmaSymbol "\\\[FormalV\]"                         conceal cchar=v
    syntax match mmaSymbol "\\\[FormalW\]"                         conceal cchar=w
    syntax match mmaSymbol "\\\[FormalX\]"                         conceal cchar=x
    syntax match mmaSymbol "\\\[FormalY\]"                         conceal cchar=y
    syntax match mmaSymbol "\\\[FormalZ\]"                         conceal cchar=z
    syntax match mmaSymbol "\\\[FormalCapitalA\]"                  conceal cchar=A
    syntax match mmaSymbol "\\\[FormalCapitalB\]"                  conceal cchar=B
    syntax match mmaSymbol "\\\[FormalCapitalC\]"                  conceal cchar=C
    syntax match mmaSymbol "\\\[FormalCapitalD\]"                  conceal cchar=D
    syntax match mmaSymbol "\\\[FormalCapitalE\]"                  conceal cchar=E
    syntax match mmaSymbol "\\\[FormalCapitalF\]"                  conceal cchar=F
    syntax match mmaSymbol "\\\[FormalCapitalG\]"                  conceal cchar=G
    syntax match mmaSymbol "\\\[FormalCapitalH\]"                  conceal cchar=H
    syntax match mmaSymbol "\\\[FormalCapitalI\]"                  conceal cchar=I
    syntax match mmaSymbol "\\\[FormalCapitalJ\]"                  conceal cchar=J
    syntax match mmaSymbol "\\\[FormalCapitalK\]"                  conceal cchar=K
    syntax match mmaSymbol "\\\[FormalCapitalL\]"                  conceal cchar=L
    syntax match mmaSymbol "\\\[FormalCapitalM\]"                  conceal cchar=M
    syntax match mmaSymbol "\\\[FormalCapitalN\]"                  conceal cchar=N
    syntax match mmaSymbol "\\\[FormalCapitalO\]"                  conceal cchar=O
    syntax match mmaSymbol "\\\[FormalCapitalP\]"                  conceal cchar=P
    syntax match mmaSymbol "\\\[FormalCapitalQ\]"                  conceal cchar=Q
    syntax match mmaSymbol "\\\[FormalCapitalR\]"                  conceal cchar=R
    syntax match mmaSymbol "\\\[FormalCapitalS\]"                  conceal cchar=S
    syntax match mmaSymbol "\\\[FormalCapitalT\]"                  conceal cchar=T
    syntax match mmaSymbol "\\\[FormalCapitalU\]"                  conceal cchar=U
    syntax match mmaSymbol "\\\[FormalCapitalV\]"                  conceal cchar=V
    syntax match mmaSymbol "\\\[FormalCapitalW\]"                  conceal cchar=W
    syntax match mmaSymbol "\\\[FormalCapitalX\]"                  conceal cchar=X
    syntax match mmaSymbol "\\\[FormalCapitalY\]"                  conceal cchar=Y
    syntax match mmaSymbol "\\\[FormalCapitalZ\]"                  conceal cchar=Z
    syntax match mmaSymbol "\\\[FormalCapitalAlpha\]"              conceal cchar=Α
    syntax match mmaSymbol "\\\[FormalCapitalBeta\]"               conceal cchar=Β
    syntax match mmaSymbol "\\\[FormalCapitalGamma\]"              conceal cchar=Γ
    syntax match mmaSymbol "\\\[FormalCapitalDelta\]"              conceal cchar=Δ
    syntax match mmaSymbol "\\\[FormalCapitalEpsilon\]"            conceal cchar=Ε
    syntax match mmaSymbol "\\\[FormalCapitalZeta\]"               conceal cchar=Ζ
    syntax match mmaSymbol "\\\[FormalCapitalEta\]"                conceal cchar=Η
    syntax match mmaSymbol "\\\[FormalCapitalTheta\]"              conceal cchar=Θ
    syntax match mmaSymbol "\\\[FormalCapitalIota\]"               conceal cchar=Ι
    syntax match mmaSymbol "\\\[FormalCapitalKappa\]"              conceal cchar=Κ
    syntax match mmaSymbol "\\\[FormalCapitalLambda\]"             conceal cchar=Λ
    syntax match mmaSymbol "\\\[FormalCapitalMu\]"                 conceal cchar=Μ
    syntax match mmaSymbol "\\\[FormalCapitalNu\]"                 conceal cchar=Ν
    syntax match mmaSymbol "\\\[FormalCapitalXi\]"                 conceal cchar=Ξ
    syntax match mmaSymbol "\\\[FormalCapitalOmicron\]"            conceal cchar=Ο
    syntax match mmaSymbol "\\\[FormalCapitalPi\]"                 conceal cchar=Π
    syntax match mmaSymbol "\\\[FormalCapitalRho\]"                conceal cchar=Ρ
    syntax match mmaSymbol "\\\[FormalCapitalSigma\]"              conceal cchar=Σ
    syntax match mmaSymbol "\\\[FormalCapitalTau\]"                conceal cchar=Τ
    syntax match mmaSymbol "\\\[FormalCapitalUpsilon\]"            conceal cchar=Υ
    syntax match mmaSymbol "\\\[FormalCapitalPhi\]"                conceal cchar=Φ
    syntax match mmaSymbol "\\\[FormalCapitalChi\]"                conceal cchar=Χ
    syntax match mmaSymbol "\\\[FormalCapitalPsi\]"                conceal cchar=Ψ
    syntax match mmaSymbol "\\\[FormalCapitalOmega\]"              conceal cchar=Ω
    syntax match mmaSymbol "\\\[FormalAlpha\]"                     conceal cchar=α
    syntax match mmaSymbol "\\\[FormalBeta\]"                      conceal cchar=β
    syntax match mmaSymbol "\\\[FormalGamma\]"                     conceal cchar=γ
    syntax match mmaSymbol "\\\[FormalDelta\]"                     conceal cchar=δ
    syntax match mmaSymbol "\\\[FormalCurlyEpsilon\]"              conceal cchar=ε
    syntax match mmaSymbol "\\\[FormalZeta\]"                      conceal cchar=ζ
    syntax match mmaSymbol "\\\[FormalEta\]"                       conceal cchar=η
    syntax match mmaSymbol "\\\[FormalTheta\]"                     conceal cchar=θ
    syntax match mmaSymbol "\\\[FormalIota\]"                      conceal cchar=ι
    syntax match mmaSymbol "\\\[FormalKappa\]"                     conceal cchar=κ
    syntax match mmaSymbol "\\\[FormalLambda\]"                    conceal cchar=λ
    syntax match mmaSymbol "\\\[FormalMu\]"                        conceal cchar=μ
    syntax match mmaSymbol "\\\[FormalNu\]"                        conceal cchar=ν
    syntax match mmaSymbol "\\\[FormalXi\]"                        conceal cchar=ξ
    syntax match mmaSymbol "\\\[FormalOmicron\]"                   conceal cchar=ο
    syntax match mmaSymbol "\\\[FormalPi\]"                        conceal cchar=π
    syntax match mmaSymbol "\\\[FormalRho\]"                       conceal cchar=ρ
    syntax match mmaSymbol "\\\[FormalFinalSigma\]"                conceal cchar=ς
    syntax match mmaSymbol "\\\[FormalSigma\]"                     conceal cchar=σ
    syntax match mmaSymbol "\\\[FormalTau\]"                       conceal cchar=τ
    syntax match mmaSymbol "\\\[FormalUpsilon\]"                   conceal cchar=υ
    syntax match mmaSymbol "\\\[FormalCurlyPhi\]"                  conceal cchar=φ
    syntax match mmaSymbol "\\\[FormalChi\]"                       conceal cchar=χ
    syntax match mmaSymbol "\\\[FormalPsi\]"                       conceal cchar=ψ
    syntax match mmaSymbol "\\\[FormalOmega\]"                     conceal cchar=ω
    syntax match mmaSymbol "\\\[FormalCurlyTheta\]"                conceal cchar=ϑ
    syntax match mmaSymbol "\\\[FormalCurlyCapitalUpsilon\]"       conceal cchar=ϒ
    syntax match mmaSymbol "\\\[FormalPhi\]"                       conceal cchar=ϕ
    syntax match mmaSymbol "\\\[FormalCurlyPi\]"                   conceal cchar=ϖ
    syntax match mmaSymbol "\\\[FormalCapitalStigma\]"             conceal cchar=Ϛ
    syntax match mmaSymbol "\\\[FormalStigma\]"                    conceal cchar=ϛ
    syntax match mmaSymbol "\\\[FormalCapitalDigamma\]"            conceal cchar=Ϝ
    syntax match mmaSymbol "\\\[FormalDigamma\]"                   conceal cchar=ϝ
    syntax match mmaSymbol "\\\[FormalCapitalKoppa\]"              conceal cchar=Ϟ
    syntax match mmaSymbol "\\\[FormalKoppa\]"                     conceal cchar=ϟ
    syntax match mmaSymbol "\\\[FormalCapitalSampi\]"              conceal cchar=Ϡ
    syntax match mmaSymbol "\\\[FormalSampi\]"                     conceal cchar=ϡ
    syntax match mmaSymbol "\\\[FormalCurlyKappa\]"                conceal cchar=ϰ
    syntax match mmaSymbol "\\\[FormalCurlyRho\]"                  conceal cchar=ϱ
    syntax match mmaSymbol "\\\[FormalEpsilon\]"                   conceal cchar=ϵ
    syntax match mmaSymbol "\\\[AE\]"                              conceal cchar=æ
    syntax match mmaSymbol "\\\[AAcute\]"                          conceal cchar=á
    syntax match mmaSymbol "\\\[ABar\]"                            conceal cchar=ā
    syntax match mmaSymbol "\\\[AGrave\]"                          conceal cchar=à
    syntax match mmaSymbol "\\\[ACup\]"                            conceal cchar=ă
    syntax match mmaSymbol "\\\[AHat\]"                            conceal cchar=â
    syntax match mmaSymbol "\\\[ARing\]"                           conceal cchar=å
    syntax match mmaSymbol "\\\[ADoubleDot\]"                      conceal cchar=ä
    syntax match mmaSymbol "\\\[ATilde\]"                          conceal cchar=ã
    syntax match mmaSymbol "\\\[CapitalAE\]"                       conceal cchar=Æ
    syntax match mmaSymbol "\\\[CapitalAAcute\]"                   conceal cchar=Á
    syntax match mmaSymbol "\\\[CapitalABar\]"                     conceal cchar=Ā
    syntax match mmaSymbol "\\\[CapitalAGrave\]"                   conceal cchar=À
    syntax match mmaSymbol "\\\[CapitalACup\]"                     conceal cchar=Ă
    syntax match mmaSymbol "\\\[CapitalAHat\]"                     conceal cchar=Â
    syntax match mmaSymbol "\\\[CapitalARing\]"                    conceal cchar=Å
    syntax match mmaSymbol "\\\[CapitalADoubleDot\]"               conceal cchar=Ä
    syntax match mmaSymbol "\\\[CapitalATilde\]"                   conceal cchar=Ã
    syntax match mmaSymbol "\\\[CAcute\]"                          conceal cchar=ć
    syntax match mmaSymbol "\\\[CHacek\]"                          conceal cchar=č
    syntax match mmaSymbol "\\\[CCedilla\]"                        conceal cchar=ç
    syntax match mmaSymbol "\\\[CapitalCAcute\]"                   conceal cchar=Ć
    syntax match mmaSymbol "\\\[CapitalCHacek\]"                   conceal cchar=Č
    syntax match mmaSymbol "\\\[CapitalCCedilla\]"                 conceal cchar=Ç
    syntax match mmaSymbol "\\\[Eth\]"                             conceal cchar=ð
    syntax match mmaSymbol "\\\[CapitalEth\]"                      conceal cchar=Ð
    syntax match mmaSymbol "\\\[DHacek\]"                          conceal cchar=ď
    syntax match mmaSymbol "\\\[CapitalDHacek\]"                   conceal cchar=Ď
    syntax match mmaSymbol "\\\[EAcute\]"                          conceal cchar=é
    syntax match mmaSymbol "\\\[EBar\]"                            conceal cchar=ē
    syntax match mmaSymbol "\\\[EGrave\]"                          conceal cchar=è
    syntax match mmaSymbol "\\\[ECup\]"                            conceal cchar=ĕ
    syntax match mmaSymbol "\\\[EHat\]"                            conceal cchar=ê
    syntax match mmaSymbol "\\\[EDoubleDot\]"                      conceal cchar=ë
    syntax match mmaSymbol "\\\[EHacek\]"                          conceal cchar=ě
    syntax match mmaSymbol "\\\[CapitalEHacek\]"                   conceal cchar=Ě
    syntax match mmaSymbol "\\\[CapitalEAcute\]"                   conceal cchar=É
    syntax match mmaSymbol "\\\[CapitalEBar\]"                     conceal cchar=Ē
    syntax match mmaSymbol "\\\[CapitalEGrave\]"                   conceal cchar=È
    syntax match mmaSymbol "\\\[CapitalECup\]"                     conceal cchar=Ĕ
    syntax match mmaSymbol "\\\[CapitalEHat\]"                     conceal cchar=Ê
    syntax match mmaSymbol "\\\[CapitalEDoubleDot\]"               conceal cchar=Ë
    syntax match mmaSymbol "\\\[FiLigature\]"                      conceal cchar=ﬁ
    syntax match mmaSymbol "\\\[FlLigature\]"                      conceal cchar=ﬂ
    syntax match mmaSymbol "\\\[DotlessI\]"                        conceal cchar=ı
    syntax match mmaSymbol "\\\[IAcute\]"                          conceal cchar=í
    syntax match mmaSymbol "\\\[IGrave\]"                          conceal cchar=ì
    syntax match mmaSymbol "\\\[ICup\]"                            conceal cchar=ĭ
    syntax match mmaSymbol "\\\[IHat\]"                            conceal cchar=î
    syntax match mmaSymbol "\\\[IDoubleDot\]"                      conceal cchar=ï
    syntax match mmaSymbol "\\\[CapitalIAcute\]"                   conceal cchar=Í
    syntax match mmaSymbol "\\\[CapitalIGrave\]"                   conceal cchar=Ì
    syntax match mmaSymbol "\\\[CapitalICup\]"                     conceal cchar=Ĭ
    syntax match mmaSymbol "\\\[CapitalIHat\]"                     conceal cchar=Î
    syntax match mmaSymbol "\\\[CapitalIDoubleDot\]"               conceal cchar=Ï
    syntax match mmaSymbol "\\\[DotlessJ\]"                        conceal cchar=
    syntax match mmaSymbol "\\\[LSlash\]"                          conceal cchar=ł
    syntax match mmaSymbol "\\\[CapitalLSlash\]"                   conceal cchar=Ł
    syntax match mmaSymbol "\\\[NTilde\]"                          conceal cchar=ñ
    syntax match mmaSymbol "\\\[CapitalNTilde\]"                   conceal cchar=Ñ
    syntax match mmaSymbol "\\\[NHacek\]"                          conceal cchar=ň
    syntax match mmaSymbol "\\\[CapitalNHacek\]"                   conceal cchar=Ň
    syntax match mmaSymbol "\\\[OAcute\]"                          conceal cchar=ó
    syntax match mmaSymbol "\\\[OE\]"                              conceal cchar=œ
    syntax match mmaSymbol "\\\[OGrave\]"                          conceal cchar=ò
    syntax match mmaSymbol "\\\[OHat\]"                            conceal cchar=ô
    syntax match mmaSymbol "\\\[ODoubleDot\]"                      conceal cchar=ö
    syntax match mmaSymbol "\\\[ODoubleAcute\]"                    conceal cchar=ő
    syntax match mmaSymbol "\\\[OTilde\]"                          conceal cchar=õ
    syntax match mmaSymbol "\\\[OSlash\]"                          conceal cchar=ø
    syntax match mmaSymbol "\\\[CapitalOAcute\]"                   conceal cchar=Ó
    syntax match mmaSymbol "\\\[CapitalOGrave\]"                   conceal cchar=Ò
    syntax match mmaSymbol "\\\[CapitalOE\]"                       conceal cchar=Œ
    syntax match mmaSymbol "\\\[CapitalOHat\]"                     conceal cchar=Ô
    syntax match mmaSymbol "\\\[CapitalODoubleDot\]"               conceal cchar=Ö
    syntax match mmaSymbol "\\\[CapitalODoubleAcute\]"             conceal cchar=Ő
    syntax match mmaSymbol "\\\[CapitalOTilde\]"                   conceal cchar=Õ
    syntax match mmaSymbol "\\\[CapitalOSlash\]"                   conceal cchar=Ø
    syntax match mmaSymbol "\\\[RHacek\]"                          conceal cchar=ř
    syntax match mmaSymbol "\\\[CapitalRHacek\]"                   conceal cchar=Ř
    syntax match mmaSymbol "\\\[SHacek\]"                          conceal cchar=š
    syntax match mmaSymbol "\\\[SZ\]"                              conceal cchar=ß
    syntax match mmaSymbol "\\\[CapitalSHacek\]"                   conceal cchar=Š
    syntax match mmaSymbol "\\\[Thorn\]"                           conceal cchar=þ
    syntax match mmaSymbol "\\\[CapitalThorn\]"                    conceal cchar=Þ
    syntax match mmaSymbol "\\\[THacek\]"                          conceal cchar=ť
    syntax match mmaSymbol "\\\[CapitalTHacek\]"                   conceal cchar=Ť
    syntax match mmaSymbol "\\\[UAcute\]"                          conceal cchar=ú
    syntax match mmaSymbol "\\\[UGrave\]"                          conceal cchar=ù
    syntax match mmaSymbol "\\\[UHat\]"                            conceal cchar=û
    syntax match mmaSymbol "\\\[UDoubleDot\]"                      conceal cchar=ü
    syntax match mmaSymbol "\\\[UDoubleAcute\]"                    conceal cchar=ű
    syntax match mmaSymbol "\\\[URing\]"                           conceal cchar=ů
    syntax match mmaSymbol "\\\[CapitalURing\]"                    conceal cchar=Ů
    syntax match mmaSymbol "\\\[CapitalUAcute\]"                   conceal cchar=Ú
    syntax match mmaSymbol "\\\[CapitalUGrave\]"                   conceal cchar=Ù
    syntax match mmaSymbol "\\\[CapitalUHat\]"                     conceal cchar=Û
    syntax match mmaSymbol "\\\[CapitalUDoubleDot\]"               conceal cchar=Ü
    syntax match mmaSymbol "\\\[CapitalUDoubleAcute\]"             conceal cchar=Ű
    syntax match mmaSymbol "\\\[YAcute\]"                          conceal cchar=ý
    syntax match mmaSymbol "\\\[YDoubleDot\]"                      conceal cchar=ÿ
    syntax match mmaSymbol "\\\[CapitalYAcute\]"                   conceal cchar=Ý
    syntax match mmaSymbol "\\\[ZHacek\]"                          conceal cchar=ž
    syntax match mmaSymbol "\\\[CapitalZHacek\]"                   conceal cchar=Ž
    syntax match mmaSymbol "\\\[ScriptA\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalA\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptB\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalB\]"                  conceal cchar=ℬ
    syntax match mmaSymbol "\\\[ScriptC\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalC\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptD\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalD\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptE\]"                         conceal cchar=ℯ
    syntax match mmaSymbol "\\\[ScriptCapitalE\]"                  conceal cchar=ℰ
    syntax match mmaSymbol "\\\[ScriptF\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalF\]"                  conceal cchar=ℱ
    syntax match mmaSymbol "\\\[ScriptG\]"                         conceal cchar=ℊ
    syntax match mmaSymbol "\\\[ScriptCapitalG\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptH\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalH\]"                  conceal cchar=ℋ
    syntax match mmaSymbol "\\\[ScriptI\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptDotlessI\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalI\]"                  conceal cchar=ℐ
    syntax match mmaSymbol "\\\[ScriptJ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptDotlessJ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalJ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptK\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalK\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptL\]"                         conceal cchar=ℓ
    syntax match mmaSymbol "\\\[ScriptCapitalL\]"                  conceal cchar=ℒ
    syntax match mmaSymbol "\\\[ScriptM\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalM\]"                  conceal cchar=ℳ
    syntax match mmaSymbol "\\\[ScriptN\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalN\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptO\]"                         conceal cchar=ℴ
    syntax match mmaSymbol "\\\[ScriptCapitalO\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptP\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalP\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptQ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalQ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptR\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalR\]"                  conceal cchar=ℛ
    syntax match mmaSymbol "\\\[ScriptS\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalS\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptT\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalT\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptU\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalU\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptV\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalV\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptW\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalW\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptX\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalX\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptY\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalY\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptZ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalZ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicA\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalA\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicB\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalB\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicC\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalC\]"                  conceal cchar=ℭ
    syntax match mmaSymbol "\\\[GothicD\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalD\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicE\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalE\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicF\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalF\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicG\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalG\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicH\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalH\]"                  conceal cchar=ℌ
    syntax match mmaSymbol "\\\[GothicI\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalI\]"                  conceal cchar=ℑ
    syntax match mmaSymbol "\\\[GothicJ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalJ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicK\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalK\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicL\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalL\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicM\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalM\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicN\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalN\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicO\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalO\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicP\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalP\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicQ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalQ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicR\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalR\]"                  conceal cchar=ℜ
    syntax match mmaSymbol "\\\[GothicS\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalS\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicT\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalT\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicU\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalU\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicV\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalV\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicW\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalW\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicX\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalX\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicY\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalY\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicZ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalZ\]"                  conceal cchar=ℨ
    syntax match mmaSymbol "\\\[DoubleStruckA\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalA\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckB\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalB\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckC\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalC\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckD\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalD\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckE\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalE\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckF\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalF\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckG\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalG\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckH\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalH\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckI\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalI\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckJ\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalJ\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckK\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalK\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckL\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalL\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckM\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalM\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckN\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalN\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckO\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalO\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckP\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalP\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckQ\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalQ\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckR\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalR\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckS\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalS\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckT\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalT\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckU\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalU\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckV\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalV\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckW\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalW\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckX\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalX\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckY\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalY\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckZ\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalZ\]"            conceal cchar=
    syntax match mmaSymbol "\\\[Alpha\]"                           conceal cchar=α
    syntax match mmaSymbol "\\\[CapitalAlpha\]"                    conceal cchar=Α
    syntax match mmaSymbol "\\\[Beta\]"                            conceal cchar=β
    syntax match mmaSymbol "\\\[CapitalBeta\]"                     conceal cchar=Β
    syntax match mmaSymbol "\\\[Gamma\]"                           conceal cchar=γ
    syntax match mmaSymbol "\\\[CapitalGamma\]"                    conceal cchar=Γ
    syntax match mmaSymbol "\\\[Delta\]"                           conceal cchar=δ
    syntax match mmaSymbol "\\\[CapitalDelta\]"                    conceal cchar=Δ
    syntax match mmaSymbol "\\\[Epsilon\]"                         conceal cchar=ϵ
    syntax match mmaSymbol "\\\[CurlyEpsilon\]"                    conceal cchar=ε
    syntax match mmaSymbol "\\\[CapitalEpsilon\]"                  conceal cchar=Ε
    syntax match mmaSymbol "\\\[Zeta\]"                            conceal cchar=ζ
    syntax match mmaSymbol "\\\[CapitalZeta\]"                     conceal cchar=Ζ
    syntax match mmaSymbol "\\\[Eta\]"                             conceal cchar=η
    syntax match mmaSymbol "\\\[CapitalEta\]"                      conceal cchar=Η
    syntax match mmaSymbol "\\\[Theta\]"                           conceal cchar=θ
    syntax match mmaSymbol "\\\[CurlyTheta\]"                      conceal cchar=ϑ
    syntax match mmaSymbol "\\\[CapitalTheta\]"                    conceal cchar=Θ
    syntax match mmaSymbol "\\\[Iota\]"                            conceal cchar=ι
    syntax match mmaSymbol "\\\[CapitalIota\]"                     conceal cchar=Ι
    syntax match mmaSymbol "\\\[Kappa\]"                           conceal cchar=κ
    syntax match mmaSymbol "\\\[CurlyKappa\]"                      conceal cchar=ϰ
    syntax match mmaSymbol "\\\[CapitalKappa\]"                    conceal cchar=Κ
    syntax match mmaSymbol "\\\[Lambda\]"                          conceal cchar=λ
    syntax match mmaSymbol "\\\[CapitalLambda\]"                   conceal cchar=Λ
    syntax match mmaSymbol "\\\[Mu\]"                              conceal cchar=μ
    syntax match mmaSymbol "\\\[CapitalMu\]"                       conceal cchar=Μ
    syntax match mmaSymbol "\\\[Nu\]"                              conceal cchar=ν
    syntax match mmaSymbol "\\\[CapitalNu\]"                       conceal cchar=Ν
    syntax match mmaSymbol "\\\[Xi\]"                              conceal cchar=ξ
    syntax match mmaSymbol "\\\[CapitalXi\]"                       conceal cchar=Ξ
    syntax match mmaSymbol "\\\[Omicron\]"                         conceal cchar=ο
    syntax match mmaSymbol "\\\[CapitalOmicron\]"                  conceal cchar=Ο
    syntax match mmaSymbol "\\\[Pi\]"                              conceal cchar=π
    syntax match mmaSymbol "\\\[CurlyPi\]"                         conceal cchar=ϖ
    syntax match mmaSymbol "\\\[CapitalPi\]"                       conceal cchar=Π
    syntax match mmaSymbol "\\\[Rho\]"                             conceal cchar=ρ
    syntax match mmaSymbol "\\\[CurlyRho\]"                        conceal cchar=ϱ
    syntax match mmaSymbol "\\\[CapitalRho\]"                      conceal cchar=Ρ
    syntax match mmaSymbol "\\\[Sigma\]"                           conceal cchar=σ
    syntax match mmaSymbol "\\\[FinalSigma\]"                      conceal cchar=ς
    syntax match mmaSymbol "\\\[CapitalSigma\]"                    conceal cchar=Σ
    syntax match mmaSymbol "\\\[Tau\]"                             conceal cchar=τ
    syntax match mmaSymbol "\\\[CapitalTau\]"                      conceal cchar=Τ
    syntax match mmaSymbol "\\\[Upsilon\]"                         conceal cchar=υ
    syntax match mmaSymbol "\\\[CapitalUpsilon\]"                  conceal cchar=Υ
    syntax match mmaSymbol "\\\[CurlyCapitalUpsilon\]"             conceal cchar=ϒ
    syntax match mmaSymbol "\\\[Phi\]"                             conceal cchar=ϕ
    syntax match mmaSymbol "\\\[CurlyPhi\]"                        conceal cchar=φ
    syntax match mmaSymbol "\\\[CapitalPhi\]"                      conceal cchar=Φ
    syntax match mmaSymbol "\\\[Chi\]"                             conceal cchar=χ
    syntax match mmaSymbol "\\\[CapitalChi\]"                      conceal cchar=Χ
    syntax match mmaSymbol "\\\[Psi\]"                             conceal cchar=ψ
    syntax match mmaSymbol "\\\[CapitalPsi\]"                      conceal cchar=Ψ
    syntax match mmaSymbol "\\\[Omega\]"                           conceal cchar=ω
    syntax match mmaSymbol "\\\[CapitalOmega\]"                    conceal cchar=Ω
    syntax match mmaSymbol "\\\[Stigma\]"                          conceal cchar=ϛ
    syntax match mmaSymbol "\\\[CapitalStigma\]"                   conceal cchar=Ϛ
    syntax match mmaSymbol "\\\[Digamma\]"                         conceal cchar=ϝ
    syntax match mmaSymbol "\\\[CapitalDigamma\]"                  conceal cchar=Ϝ
    syntax match mmaSymbol "\\\[Koppa\]"                           conceal cchar=ϟ
    syntax match mmaSymbol "\\\[CapitalKoppa\]"                    conceal cchar=Ϟ
    syntax match mmaSymbol "\\\[Sampi\]"                           conceal cchar=ϡ
    syntax match mmaSymbol "\\\[CapitalSampi\]"                    conceal cchar=Ϡ
    syntax match mmaSymbol "\\\[Aleph\]"                           conceal cchar=ℵ
    syntax match mmaSymbol "\\\[Bet\]"                             conceal cchar=ℶ
    syntax match mmaSymbol "\\\[Gimel\]"                           conceal cchar=ℷ
    syntax match mmaSymbol "\\\[Dalet\]"                           conceal cchar=ℸ
    syntax match mmaSymbol "\\\[Hacek\]"                           conceal cchar=ˇ
    syntax match mmaSymbol "\\\[Florin\]"                          conceal cchar=ƒ
    syntax match mmaSymbol "\\\[Micro\]"                           conceal cchar=µ
    syntax match mmaSymbol "\\\[AAcute\]"                          conceal cchar=á
    syntax match mmaSymbol "\\\[ABar\]"                            conceal cchar=ā
    syntax match mmaSymbol "\\\[ACup\]"                            conceal cchar=ă
    syntax match mmaSymbol "\\\[ADoubleDot\]"                      conceal cchar=ä
    syntax match mmaSymbol "\\\[AE\]"                              conceal cchar=æ
    syntax match mmaSymbol "\\\[AGrave\]"                          conceal cchar=à
    syntax match mmaSymbol "\\\[AHat\]"                            conceal cchar=â
    syntax match mmaSymbol "\\\[Aleph\]"                           conceal cchar=ℵ
    syntax match mmaSymbol "\\\[Alpha\]"                           conceal cchar=α
    syntax match mmaSymbol "\\\[AltKey\]"                          conceal cchar=
    syntax match mmaSymbol "\\\[And\]"                             conceal cchar=∧
    syntax match mmaSymbol "\\\[Angle\]"                           conceal cchar=∠
    syntax match mmaSymbol "\\\[Angstrom\]"                        conceal cchar=Å
    syntax match mmaSymbol "\\\[AquariusSign\]"                    conceal cchar=♒
    syntax match mmaSymbol "\\\[AriesSign\]"                       conceal cchar=♈
    syntax match mmaSymbol "\\\[ARing\]"                           conceal cchar=å
    syntax match mmaSymbol "\\\[AscendingEllipsis\]"               conceal cchar=⋰
    syntax match mmaSymbol "\\\[ATilde\]"                          conceal cchar=ã
    syntax match mmaSymbol "\\\[AutoPlaceholder\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[Backslash\]"                       conceal cchar=∖
    syntax match mmaSymbol "\\\[BeamedEighthNote\]"                conceal cchar=♫
    syntax match mmaSymbol "\\\[BeamedSixteenthNote\]"             conceal cchar=♬
    syntax match mmaSymbol "\\\[Because\]"                         conceal cchar=∵
    syntax match mmaSymbol "\\\[Bet\]"                             conceal cchar=ℶ
    syntax match mmaSymbol "\\\[Beta\]"                            conceal cchar=β
    syntax match mmaSymbol "\\\[BlackBishop\]"                     conceal cchar=♝
    syntax match mmaSymbol "\\\[BlackKing\]"                       conceal cchar=♚
    syntax match mmaSymbol "\\\[BlackKnight\]"                     conceal cchar=♞
    syntax match mmaSymbol "\\\[BlackPawn\]"                       conceal cchar=♟
    syntax match mmaSymbol "\\\[BlackQueen\]"                      conceal cchar=♛
    syntax match mmaSymbol "\\\[BlackRook\]"                       conceal cchar=♜
    syntax match mmaSymbol "\\\[Breve\]"                           conceal cchar=˘
    syntax match mmaSymbol "\\\[Bullet\]"                          conceal cchar=•
    syntax match mmaSymbol "\\\[CAcute\]"                          conceal cchar=ć
    syntax match mmaSymbol "\\\[CancerSign\]"                      conceal cchar=♋
    syntax match mmaSymbol "\\\[Cap\]"                             conceal cchar=⌢
    syntax match mmaSymbol "\\\[CapitalAAcute\]"                   conceal cchar=Á
    syntax match mmaSymbol "\\\[CapitalABar\]"                     conceal cchar=Ā
    syntax match mmaSymbol "\\\[CapitalACup\]"                     conceal cchar=Ă
    syntax match mmaSymbol "\\\[CapitalADoubleDot\]"               conceal cchar=Ä
    syntax match mmaSymbol "\\\[CapitalAE\]"                       conceal cchar=Æ
    syntax match mmaSymbol "\\\[CapitalAGrave\]"                   conceal cchar=À
    syntax match mmaSymbol "\\\[CapitalAHat\]"                     conceal cchar=Â
    syntax match mmaSymbol "\\\[CapitalAlpha\]"                    conceal cchar=Α
    syntax match mmaSymbol "\\\[CapitalARing\]"                    conceal cchar=Å
    syntax match mmaSymbol "\\\[CapitalATilde\]"                   conceal cchar=Ã
    syntax match mmaSymbol "\\\[CapitalBeta\]"                     conceal cchar=Β
    syntax match mmaSymbol "\\\[CapitalCAcute\]"                   conceal cchar=Ć
    syntax match mmaSymbol "\\\[CapitalCCedilla\]"                 conceal cchar=Ç
    syntax match mmaSymbol "\\\[CapitalCHacek\]"                   conceal cchar=Č
    syntax match mmaSymbol "\\\[CapitalChi\]"                      conceal cchar=Χ
    syntax match mmaSymbol "\\\[CapitalDelta\]"                    conceal cchar=Δ
    syntax match mmaSymbol "\\\[CapitalDHacek\]"                   conceal cchar=Ď
    syntax match mmaSymbol "\\\[CapitalDifferentialD\]"            conceal cchar=
    syntax match mmaSymbol "\\\[CapitalDigamma\]"                  conceal cchar=Ϝ
    syntax match mmaSymbol "\\\[CapitalEAcute\]"                   conceal cchar=É
    syntax match mmaSymbol "\\\[CapitalEBar\]"                     conceal cchar=Ē
    syntax match mmaSymbol "\\\[CapitalECup\]"                     conceal cchar=Ĕ
    syntax match mmaSymbol "\\\[CapitalEDoubleDot\]"               conceal cchar=Ë
    syntax match mmaSymbol "\\\[CapitalEGrave\]"                   conceal cchar=È
    syntax match mmaSymbol "\\\[CapitalEHacek\]"                   conceal cchar=Ě
    syntax match mmaSymbol "\\\[CapitalEHat\]"                     conceal cchar=Ê
    syntax match mmaSymbol "\\\[CapitalEpsilon\]"                  conceal cchar=Ε
    syntax match mmaSymbol "\\\[CapitalEta\]"                      conceal cchar=Η
    syntax match mmaSymbol "\\\[CapitalEth\]"                      conceal cchar=Ð
    syntax match mmaSymbol "\\\[CapitalGamma\]"                    conceal cchar=Γ
    syntax match mmaSymbol "\\\[CapitalIAcute\]"                   conceal cchar=Í
    syntax match mmaSymbol "\\\[CapitalICup\]"                     conceal cchar=Ĭ
    syntax match mmaSymbol "\\\[CapitalIDoubleDot\]"               conceal cchar=Ï
    syntax match mmaSymbol "\\\[CapitalIGrave\]"                   conceal cchar=Ì
    syntax match mmaSymbol "\\\[CapitalIHat\]"                     conceal cchar=Î
    syntax match mmaSymbol "\\\[CapitalIota\]"                     conceal cchar=Ι
    syntax match mmaSymbol "\\\[CapitalKappa\]"                    conceal cchar=Κ
    syntax match mmaSymbol "\\\[CapitalKoppa\]"                    conceal cchar=Ϟ
    syntax match mmaSymbol "\\\[CapitalLambda\]"                   conceal cchar=Λ
    syntax match mmaSymbol "\\\[CapitalLSlash\]"                   conceal cchar=Ł
    syntax match mmaSymbol "\\\[CapitalMu\]"                       conceal cchar=Μ
    syntax match mmaSymbol "\\\[CapitalNHacek\]"                   conceal cchar=Ň
    syntax match mmaSymbol "\\\[CapitalNTilde\]"                   conceal cchar=Ñ
    syntax match mmaSymbol "\\\[CapitalNu\]"                       conceal cchar=Ν
    syntax match mmaSymbol "\\\[CapitalOAcute\]"                   conceal cchar=Ó
    syntax match mmaSymbol "\\\[CapitalODoubleAcute\]"             conceal cchar=Ő
    syntax match mmaSymbol "\\\[CapitalODoubleDot\]"               conceal cchar=Ö
    syntax match mmaSymbol "\\\[CapitalOE\]"                       conceal cchar=Œ
    syntax match mmaSymbol "\\\[CapitalOGrave\]"                   conceal cchar=Ò
    syntax match mmaSymbol "\\\[CapitalOHat\]"                     conceal cchar=Ô
    syntax match mmaSymbol "\\\[CapitalOmega\]"                    conceal cchar=Ω
    syntax match mmaSymbol "\\\[CapitalOmicron\]"                  conceal cchar=Ο
    syntax match mmaSymbol "\\\[CapitalOSlash\]"                   conceal cchar=Ø
    syntax match mmaSymbol "\\\[CapitalOTilde\]"                   conceal cchar=Õ
    syntax match mmaSymbol "\\\[CapitalPhi\]"                      conceal cchar=Φ
    syntax match mmaSymbol "\\\[CapitalPi\]"                       conceal cchar=Π
    syntax match mmaSymbol "\\\[CapitalPsi\]"                      conceal cchar=Ψ
    syntax match mmaSymbol "\\\[CapitalRHacek\]"                   conceal cchar=Ř
    syntax match mmaSymbol "\\\[CapitalRho\]"                      conceal cchar=Ρ
    syntax match mmaSymbol "\\\[CapitalSampi\]"                    conceal cchar=Ϡ
    syntax match mmaSymbol "\\\[CapitalSHacek\]"                   conceal cchar=Š
    syntax match mmaSymbol "\\\[CapitalSigma\]"                    conceal cchar=Σ
    syntax match mmaSymbol "\\\[CapitalStigma\]"                   conceal cchar=Ϛ
    syntax match mmaSymbol "\\\[CapitalTau\]"                      conceal cchar=Τ
    syntax match mmaSymbol "\\\[CapitalTHacek\]"                   conceal cchar=Ť
    syntax match mmaSymbol "\\\[CapitalTheta\]"                    conceal cchar=Θ
    syntax match mmaSymbol "\\\[CapitalThorn\]"                    conceal cchar=Þ
    syntax match mmaSymbol "\\\[CapitalUAcute\]"                   conceal cchar=Ú
    syntax match mmaSymbol "\\\[CapitalUDoubleAcute\]"             conceal cchar=Ű
    syntax match mmaSymbol "\\\[CapitalUDoubleDot\]"               conceal cchar=Ü
    syntax match mmaSymbol "\\\[CapitalUGrave\]"                   conceal cchar=Ù
    syntax match mmaSymbol "\\\[CapitalUHat\]"                     conceal cchar=Û
    syntax match mmaSymbol "\\\[CapitalUpsilon\]"                  conceal cchar=Υ
    syntax match mmaSymbol "\\\[CapitalURing\]"                    conceal cchar=Ů
    syntax match mmaSymbol "\\\[CapitalXi\]"                       conceal cchar=Ξ
    syntax match mmaSymbol "\\\[CapitalYAcute\]"                   conceal cchar=Ý
    syntax match mmaSymbol "\\\[CapitalZeta\]"                     conceal cchar=Ζ
    syntax match mmaSymbol "\\\[CapitalZHacek\]"                   conceal cchar=Ž
    syntax match mmaSymbol "\\\[CapricornSign\]"                   conceal cchar=♑
    syntax match mmaSymbol "\\\[CCedilla\]"                        conceal cchar=ç
    syntax match mmaSymbol "\\\[Cedilla\]"                         conceal cchar=¸
    syntax match mmaSymbol "\\\[CenterDot\]"                       conceal cchar=·
    syntax match mmaSymbol "\\\[CenterEllipsis\]"                  conceal cchar=⋯
    syntax match mmaSymbol "\\\[Cent\]"                            conceal cchar=¢
    syntax match mmaSymbol "\\\[CHacek\]"                          conceal cchar=č
    syntax match mmaSymbol "\\\[CheckedBox\]"                      conceal cchar=☒
    syntax match mmaSymbol "\\\[Checkmark\]"                       conceal cchar=✓
    syntax match mmaSymbol "\\\[Chi\]"                             conceal cchar=χ
    syntax match mmaSymbol "\\\[CircleDot\]"                       conceal cchar=⊙
    syntax match mmaSymbol "\\\[CircleMinus\]"                     conceal cchar=⊖
    syntax match mmaSymbol "\\\[CirclePlus\]"                      conceal cchar=⊕
    syntax match mmaSymbol "\\\[CircleTimes\]"                     conceal cchar=⊗
    syntax match mmaSymbol "\\\[ClockwiseContourIntegral\]"        conceal cchar=∲
    syntax match mmaSymbol "\\\[CloseCurlyDoubleQuote\]"           conceal cchar=”
    syntax match mmaSymbol "\\\[CloseCurlyQuote\]"                 conceal cchar=’
    syntax match mmaSymbol "\\\[CloverLeaf\]"                      conceal cchar=⌘
    syntax match mmaSymbol "\\\[ClubSuit\]"                        conceal cchar=♣
    syntax match mmaSymbol "\\\[Colon\]"                           conceal cchar=∶
    syntax match mmaSymbol "\\\[CommandKey\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[Conditioned\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[Congruent\]"                       conceal cchar=≡
    syntax match mmaSymbol "\\\[Conjugate\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[ConjugateTranspose\]"              conceal cchar=
    syntax match mmaSymbol "\\\[ConstantC\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[Continuation\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[ContourIntegral\]"                 conceal cchar=∮
    syntax match mmaSymbol "\\\[ControlKey\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[Coproduct\]"                       conceal cchar=∐
    syntax match mmaSymbol "\\\[Copyright\]"                       conceal cchar=©
    syntax match mmaSymbol "\\\[CounterClockwiseContourIntegral\]" conceal cchar=∳
    syntax match mmaSymbol "\\\[Cross\]"                           conceal cchar=
    syntax match mmaSymbol "\\\[CupCap\]"                          conceal cchar=≍
    syntax match mmaSymbol "\\\[Cup\]"                             conceal cchar=⌣
    syntax match mmaSymbol "\\\[CurlyCapitalUpsilon\]"             conceal cchar=ϒ
    syntax match mmaSymbol "\\\[CurlyEpsilon\]"                    conceal cchar=ε
    syntax match mmaSymbol "\\\[CurlyKappa\]"                      conceal cchar=ϰ
    syntax match mmaSymbol "\\\[CurlyPhi\]"                        conceal cchar=φ
    syntax match mmaSymbol "\\\[CurlyPi\]"                         conceal cchar=ϖ
    syntax match mmaSymbol "\\\[CurlyRho\]"                        conceal cchar=ϱ
    syntax match mmaSymbol "\\\[CurlyTheta\]"                      conceal cchar=ϑ
    syntax match mmaSymbol "\\\[Currency\]"                        conceal cchar=¤
    syntax match mmaSymbol "\\\[Dagger\]"                          conceal cchar=†
    syntax match mmaSymbol "\\\[Dalet\]"                           conceal cchar=ℸ
    syntax match mmaSymbol "\\\[Dash\]"                            conceal cchar=–
    syntax match mmaSymbol "\\\[Degree\]"                          conceal cchar=°
    syntax match mmaSymbol "\\\[DeleteKey\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[Del\]"                             conceal cchar=∇
    syntax match mmaSymbol "\\\[Delta\]"                           conceal cchar=δ
    syntax match mmaSymbol "\\\[DescendingEllipsis\]"              conceal cchar=⋱
    syntax match mmaSymbol "\\\[DHacek\]"                          conceal cchar=ď
    syntax match mmaSymbol "\\\[Diameter\]"                        conceal cchar=⌀
    syntax match mmaSymbol "\\\[Diamond\]"                         conceal cchar=⋄
    syntax match mmaSymbol "\\\[DiamondSuit\]"                     conceal cchar=♢
    syntax match mmaSymbol "\\\[DifferenceDelta\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[DifferentialD\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[Digamma\]"                         conceal cchar=ϝ
    syntax match mmaSymbol "\\\[DirectedEdge\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[DiscreteRatio\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DiscreteShift\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DiscretionaryPageBreakAbove\]"     conceal cchar=
    syntax match mmaSymbol "\\\[DiscretionaryPageBreakBelow\]"     conceal cchar=
    syntax match mmaSymbol "\\\[Distributed\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[Divides\]"                         conceal cchar=∣
    syntax match mmaSymbol "\\\[Divide\]"                          conceal cchar=÷
    syntax match mmaSymbol "\\\[DotEqual\]"                        conceal cchar=≐
    syntax match mmaSymbol "\\\[DotlessI\]"                        conceal cchar=ı
    syntax match mmaSymbol "\\\[DotlessJ\]"                        conceal cchar=
    syntax match mmaSymbol "\\\[DottedSquare\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[DoubleContourIntegral\]"           conceal cchar=∯
    syntax match mmaSymbol "\\\[DoubleDagger\]"                    conceal cchar=‡
    syntax match mmaSymbol "\\\[DoubledGamma\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[DoubleDot\]"                       conceal cchar=¨
    syntax match mmaSymbol "\\\[DoubleDownArrow\]"                 conceal cchar=⇓
    syntax match mmaSymbol "\\\[DoubledPi\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[DoubleLeftArrow\]"                 conceal cchar=⇐
    syntax match mmaSymbol "\\\[DoubleLeftRightArrow\]"            conceal cchar=⇔
    syntax match mmaSymbol "\\\[DoubleLeftTee\]"                   conceal cchar=⫤
    syntax match mmaSymbol "\\\[DoubleLongLeftArrow\]"             conceal cchar=⟸
    syntax match mmaSymbol "\\\[DoubleLongLeftRightArrow\]"        conceal cchar=⟺
    syntax match mmaSymbol "\\\[DoubleLongRightArrow\]"            conceal cchar=⟹
    syntax match mmaSymbol "\\\[DoublePrime\]"                     conceal cchar=″
    syntax match mmaSymbol "\\\[DoubleRightArrow\]"                conceal cchar=⇒
    syntax match mmaSymbol "\\\[DoubleRightTee\]"                  conceal cchar=⊨
    syntax match mmaSymbol "\\\[DoubleStruckA\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckB\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckC\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalA\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalB\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalC\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalD\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalE\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalF\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalG\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalH\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalI\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalJ\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalK\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalL\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalM\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalN\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalO\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalP\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalQ\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalR\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalS\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalT\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalU\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalV\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalW\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalX\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalY\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckCapitalZ\]"            conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckD\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckE\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckEight\]"               conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckF\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckFive\]"                conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckFour\]"                conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckG\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckH\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckI\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckJ\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckK\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckL\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckM\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckN\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckNine\]"                conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckO\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckOne\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckP\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckQ\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckR\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckS\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckSeven\]"               conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckSix\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckT\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckThree\]"               conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckTwo\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckU\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckV\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckW\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckX\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckY\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckZ\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[DoubleStruckZero\]"                conceal cchar=
    syntax match mmaSymbol "\\\[DoubleUpArrow\]"                   conceal cchar=⇑
    syntax match mmaSymbol "\\\[DoubleUpDownArrow\]"               conceal cchar=⇕
    syntax match mmaSymbol "\\\[DoubleVerticalBar\]"               conceal cchar=∥
    syntax match mmaSymbol "\\\[DownArrowBar\]"                    conceal cchar=⤓
    syntax match mmaSymbol "\\\[DownArrow\]"                       conceal cchar=↓
    syntax match mmaSymbol "\\\[DownArrowUpArrow\]"                conceal cchar=⇵
    syntax match mmaSymbol "\\\[DownBreve\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[DownExclamation\]"                 conceal cchar=¡
    syntax match mmaSymbol "\\\[DownLeftRightVector\]"             conceal cchar=⥐
    syntax match mmaSymbol "\\\[DownLeftTeeVector\]"               conceal cchar=⥞
    syntax match mmaSymbol "\\\[DownLeftVector\]"                  conceal cchar=↽
    syntax match mmaSymbol "\\\[DownLeftVectorBar\]"               conceal cchar=⥖
    syntax match mmaSymbol "\\\[DownPointer\]"                     conceal cchar=▾
    syntax match mmaSymbol "\\\[DownQuestion\]"                    conceal cchar=¿
    syntax match mmaSymbol "\\\[DownRightTeeVector\]"              conceal cchar=⥟
    syntax match mmaSymbol "\\\[DownRightVector\]"                 conceal cchar=⇁
    syntax match mmaSymbol "\\\[DownRightVectorBar\]"              conceal cchar=⥗
    syntax match mmaSymbol "\\\[DownTeeArrow\]"                    conceal cchar=↧
    syntax match mmaSymbol "\\\[DownTee\]"                         conceal cchar=⊤
    syntax match mmaSymbol "\\\[EAcute\]"                          conceal cchar=é
    syntax match mmaSymbol "\\\[Earth\]"                           conceal cchar=
    syntax match mmaSymbol "\\\[EBar\]"                            conceal cchar=ē
    syntax match mmaSymbol "\\\[ECup\]"                            conceal cchar=ĕ
    syntax match mmaSymbol "\\\[EDoubleDot\]"                      conceal cchar=ë
    syntax match mmaSymbol "\\\[EGrave\]"                          conceal cchar=è
    syntax match mmaSymbol "\\\[EHacek\]"                          conceal cchar=ě
    syntax match mmaSymbol "\\\[EHat\]"                            conceal cchar=ê
    syntax match mmaSymbol "\\\[EighthNote\]"                      conceal cchar=♪
    syntax match mmaSymbol "\\\[Element\]"                         conceal cchar=∈
    syntax match mmaSymbol "\\\[Ellipsis\]"                        conceal cchar=…
    syntax match mmaSymbol "\\\[EmptyCircle\]"                     conceal cchar=○
    syntax match mmaSymbol "\\\[EmptyDiamond\]"                    conceal cchar=◇
    syntax match mmaSymbol "\\\[EmptyDownTriangle\]"               conceal cchar=▽
    syntax match mmaSymbol "\\\[EmptyRectangle\]"                  conceal cchar=▯
    syntax match mmaSymbol "\\\[EmptySet\]"                        conceal cchar=∅
    syntax match mmaSymbol "\\\[EmptySmallCircle\]"                conceal cchar=◦
    syntax match mmaSymbol "\\\[EmptySmallSquare\]"                conceal cchar=◻
    syntax match mmaSymbol "\\\[EmptySquare\]"                     conceal cchar=□
    syntax match mmaSymbol "\\\[EmptyUpTriangle\]"                 conceal cchar=△
    syntax match mmaSymbol "\\\[EmptyVerySmallSquare\]"            conceal cchar=▫
    syntax match mmaSymbol "\\\[EnterKey\]"                        conceal cchar=
    syntax match mmaSymbol "\\\[EntityEnd\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[EntityStart\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[Epsilon\]"                         conceal cchar=ϵ
    syntax match mmaSymbol "\\\[Equal\]"                           conceal cchar=
    syntax match mmaSymbol "\\\[EqualTilde\]"                      conceal cchar=≂
    syntax match mmaSymbol "\\\[Equilibrium\]"                     conceal cchar=⇌
    syntax match mmaSymbol "\\\[Equivalent\]"                      conceal cchar=⧦
    syntax match mmaSymbol "\\\[ErrorIndicator\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[EscapeKey\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[Eta\]"                             conceal cchar=η
    syntax match mmaSymbol "\\\[Eth\]"                             conceal cchar=ð
    syntax match mmaSymbol "\\\[Euro\]"                            conceal cchar=€
    syntax match mmaSymbol "\\\[Exists\]"                          conceal cchar=∃
    syntax match mmaSymbol "\\\[ExponentialE\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[FiLigature\]"                      conceal cchar=ﬁ
    syntax match mmaSymbol "\\\[FilledCircle\]"                    conceal cchar=●
    syntax match mmaSymbol "\\\[FilledDiamond\]"                   conceal cchar=◆
    syntax match mmaSymbol "\\\[FilledDownTriangle\]"              conceal cchar=▼
    syntax match mmaSymbol "\\\[FilledLeftTriangle\]"              conceal cchar=◀
    syntax match mmaSymbol "\\\[FilledRectangle\]"                 conceal cchar=▮
    syntax match mmaSymbol "\\\[FilledRightTriangle\]"             conceal cchar=▶
    syntax match mmaSymbol "\\\[FilledSmallCircle\]"               conceal cchar=
    syntax match mmaSymbol "\\\[FilledSmallSquare\]"               conceal cchar=■
    syntax match mmaSymbol "\\\[FilledSquare\]"                    conceal cchar=■
    syntax match mmaSymbol "\\\[FilledUpTriangle\]"                conceal cchar=▲
    syntax match mmaSymbol "\\\[FilledVerySmallSquare\]"           conceal cchar=▪
    syntax match mmaSymbol "\\\[FinalSigma\]"                      conceal cchar=ς
    syntax match mmaSymbol "\\\[FirstPage\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[FivePointedStar\]"                 conceal cchar=★
    syntax match mmaSymbol "\\\[Flat\]"                            conceal cchar=♭
    syntax match mmaSymbol "\\\[FlLigature\]"                      conceal cchar=ﬂ
    syntax match mmaSymbol "\\\[Florin\]"                          conceal cchar=ƒ
    syntax match mmaSymbol "\\\[ForAll\]"                          conceal cchar=∀
    syntax match mmaSymbol "\\\[FormalA\]"                         conceal cchar=a
    syntax match mmaSymbol "\\\[FormalAlpha\]"                     conceal cchar=α
    syntax match mmaSymbol "\\\[FormalB\]"                         conceal cchar=b
    syntax match mmaSymbol "\\\[FormalBeta\]"                      conceal cchar=β
    syntax match mmaSymbol "\\\[FormalC\]"                         conceal cchar=c
    syntax match mmaSymbol "\\\[FormalCapitalA\]"                  conceal cchar=A
    syntax match mmaSymbol "\\\[FormalCapitalAlpha\]"              conceal cchar=Α
    syntax match mmaSymbol "\\\[FormalCapitalB\]"                  conceal cchar=B
    syntax match mmaSymbol "\\\[FormalCapitalBeta\]"               conceal cchar=Β
    syntax match mmaSymbol "\\\[FormalCapitalC\]"                  conceal cchar=C
    syntax match mmaSymbol "\\\[FormalCapitalChi\]"                conceal cchar=Χ
    syntax match mmaSymbol "\\\[FormalCapitalD\]"                  conceal cchar=D
    syntax match mmaSymbol "\\\[FormalCapitalDelta\]"              conceal cchar=Δ
    syntax match mmaSymbol "\\\[FormalCapitalDigamma\]"            conceal cchar=Ϝ
    syntax match mmaSymbol "\\\[FormalCapitalE\]"                  conceal cchar=E
    syntax match mmaSymbol "\\\[FormalCapitalEpsilon\]"            conceal cchar=Ε
    syntax match mmaSymbol "\\\[FormalCapitalEta\]"                conceal cchar=Η
    syntax match mmaSymbol "\\\[FormalCapitalF\]"                  conceal cchar=F
    syntax match mmaSymbol "\\\[FormalCapitalG\]"                  conceal cchar=G
    syntax match mmaSymbol "\\\[FormalCapitalGamma\]"              conceal cchar=Γ
    syntax match mmaSymbol "\\\[FormalCapitalH\]"                  conceal cchar=H
    syntax match mmaSymbol "\\\[FormalCapitalI\]"                  conceal cchar=I
    syntax match mmaSymbol "\\\[FormalCapitalIota\]"               conceal cchar=Ι
    syntax match mmaSymbol "\\\[FormalCapitalJ\]"                  conceal cchar=J
    syntax match mmaSymbol "\\\[FormalCapitalK\]"                  conceal cchar=K
    syntax match mmaSymbol "\\\[FormalCapitalKappa\]"              conceal cchar=Κ
    syntax match mmaSymbol "\\\[FormalCapitalKoppa\]"              conceal cchar=Ϟ
    syntax match mmaSymbol "\\\[FormalCapitalL\]"                  conceal cchar=L
    syntax match mmaSymbol "\\\[FormalCapitalLambda\]"             conceal cchar=Λ
    syntax match mmaSymbol "\\\[FormalCapitalM\]"                  conceal cchar=M
    syntax match mmaSymbol "\\\[FormalCapitalMu\]"                 conceal cchar=Μ
    syntax match mmaSymbol "\\\[FormalCapitalN\]"                  conceal cchar=N
    syntax match mmaSymbol "\\\[FormalCapitalNu\]"                 conceal cchar=Ν
    syntax match mmaSymbol "\\\[FormalCapitalO\]"                  conceal cchar=O
    syntax match mmaSymbol "\\\[FormalCapitalOmega\]"              conceal cchar=Ω
    syntax match mmaSymbol "\\\[FormalCapitalOmicron\]"            conceal cchar=Ο
    syntax match mmaSymbol "\\\[FormalCapitalP\]"                  conceal cchar=P
    syntax match mmaSymbol "\\\[FormalCapitalPhi\]"                conceal cchar=Φ
    syntax match mmaSymbol "\\\[FormalCapitalPi\]"                 conceal cchar=Π
    syntax match mmaSymbol "\\\[FormalCapitalPsi\]"                conceal cchar=Ψ
    syntax match mmaSymbol "\\\[FormalCapitalQ\]"                  conceal cchar=Q
    syntax match mmaSymbol "\\\[FormalCapitalR\]"                  conceal cchar=R
    syntax match mmaSymbol "\\\[FormalCapitalRho\]"                conceal cchar=Ρ
    syntax match mmaSymbol "\\\[FormalCapitalS\]"                  conceal cchar=S
    syntax match mmaSymbol "\\\[FormalCapitalSampi\]"              conceal cchar=Ϡ
    syntax match mmaSymbol "\\\[FormalCapitalSigma\]"              conceal cchar=Σ
    syntax match mmaSymbol "\\\[FormalCapitalStigma\]"             conceal cchar=Ϛ
    syntax match mmaSymbol "\\\[FormalCapitalT\]"                  conceal cchar=T
    syntax match mmaSymbol "\\\[FormalCapitalTau\]"                conceal cchar=Τ
    syntax match mmaSymbol "\\\[FormalCapitalTheta\]"              conceal cchar=Θ
    syntax match mmaSymbol "\\\[FormalCapitalU\]"                  conceal cchar=U
    syntax match mmaSymbol "\\\[FormalCapitalUpsilon\]"            conceal cchar=Υ
    syntax match mmaSymbol "\\\[FormalCapitalV\]"                  conceal cchar=V
    syntax match mmaSymbol "\\\[FormalCapitalW\]"                  conceal cchar=W
    syntax match mmaSymbol "\\\[FormalCapitalX\]"                  conceal cchar=X
    syntax match mmaSymbol "\\\[FormalCapitalXi\]"                 conceal cchar=Ξ
    syntax match mmaSymbol "\\\[FormalCapitalY\]"                  conceal cchar=Y
    syntax match mmaSymbol "\\\[FormalCapitalZ\]"                  conceal cchar=Z
    syntax match mmaSymbol "\\\[FormalCapitalZeta\]"               conceal cchar=Ζ
    syntax match mmaSymbol "\\\[FormalChi\]"                       conceal cchar=χ
    syntax match mmaSymbol "\\\[FormalCurlyCapitalUpsilon\]"       conceal cchar=ϒ
    syntax match mmaSymbol "\\\[FormalCurlyEpsilon\]"              conceal cchar=ε
    syntax match mmaSymbol "\\\[FormalCurlyKappa\]"                conceal cchar=ϰ
    syntax match mmaSymbol "\\\[FormalCurlyPhi\]"                  conceal cchar=φ
    syntax match mmaSymbol "\\\[FormalCurlyPi\]"                   conceal cchar=ϖ
    syntax match mmaSymbol "\\\[FormalCurlyRho\]"                  conceal cchar=ϱ
    syntax match mmaSymbol "\\\[FormalCurlyTheta\]"                conceal cchar=ϑ
    syntax match mmaSymbol "\\\[FormalD\]"                         conceal cchar=d
    syntax match mmaSymbol "\\\[FormalDelta\]"                     conceal cchar=δ
    syntax match mmaSymbol "\\\[FormalDigamma\]"                   conceal cchar=ϝ
    syntax match mmaSymbol "\\\[FormalE\]"                         conceal cchar=e
    syntax match mmaSymbol "\\\[FormalEpsilon\]"                   conceal cchar=ϵ
    syntax match mmaSymbol "\\\[FormalEta\]"                       conceal cchar=η
    syntax match mmaSymbol "\\\[FormalF\]"                         conceal cchar=f
    syntax match mmaSymbol "\\\[FormalFinalSigma\]"                conceal cchar=ς
    syntax match mmaSymbol "\\\[FormalG\]"                         conceal cchar=g
    syntax match mmaSymbol "\\\[FormalGamma\]"                     conceal cchar=γ
    syntax match mmaSymbol "\\\[FormalH\]"                         conceal cchar=h
    syntax match mmaSymbol "\\\[FormalI\]"                         conceal cchar=i
    syntax match mmaSymbol "\\\[FormalIota\]"                      conceal cchar=ι
    syntax match mmaSymbol "\\\[FormalJ\]"                         conceal cchar=j
    syntax match mmaSymbol "\\\[FormalK\]"                         conceal cchar=k
    syntax match mmaSymbol "\\\[FormalKappa\]"                     conceal cchar=κ
    syntax match mmaSymbol "\\\[FormalKoppa\]"                     conceal cchar=ϟ
    syntax match mmaSymbol "\\\[FormalL\]"                         conceal cchar=l
    syntax match mmaSymbol "\\\[FormalLambda\]"                    conceal cchar=λ
    syntax match mmaSymbol "\\\[FormalM\]"                         conceal cchar=m
    syntax match mmaSymbol "\\\[FormalMu\]"                        conceal cchar=μ
    syntax match mmaSymbol "\\\[FormalN\]"                         conceal cchar=n
    syntax match mmaSymbol "\\\[FormalNu\]"                        conceal cchar=ν
    syntax match mmaSymbol "\\\[FormalO\]"                         conceal cchar=o
    syntax match mmaSymbol "\\\[FormalOmega\]"                     conceal cchar=ω
    syntax match mmaSymbol "\\\[FormalOmicron\]"                   conceal cchar=ο
    syntax match mmaSymbol "\\\[FormalP\]"                         conceal cchar=p
    syntax match mmaSymbol "\\\[FormalPhi\]"                       conceal cchar=ϕ
    syntax match mmaSymbol "\\\[FormalPi\]"                        conceal cchar=π
    syntax match mmaSymbol "\\\[FormalPsi\]"                       conceal cchar=ψ
    syntax match mmaSymbol "\\\[FormalQ\]"                         conceal cchar=q
    syntax match mmaSymbol "\\\[FormalR\]"                         conceal cchar=r
    syntax match mmaSymbol "\\\[FormalRho\]"                       conceal cchar=ρ
    syntax match mmaSymbol "\\\[FormalS\]"                         conceal cchar=s
    syntax match mmaSymbol "\\\[FormalSampi\]"                     conceal cchar=ϡ
    syntax match mmaSymbol "\\\[FormalSigma\]"                     conceal cchar=σ
    syntax match mmaSymbol "\\\[FormalStigma\]"                    conceal cchar=ϛ
    syntax match mmaSymbol "\\\[FormalT\]"                         conceal cchar=t
    syntax match mmaSymbol "\\\[FormalTau\]"                       conceal cchar=τ
    syntax match mmaSymbol "\\\[FormalTheta\]"                     conceal cchar=θ
    syntax match mmaSymbol "\\\[FormalU\]"                         conceal cchar=u
    syntax match mmaSymbol "\\\[FormalUpsilon\]"                   conceal cchar=υ
    syntax match mmaSymbol "\\\[FormalV\]"                         conceal cchar=v
    syntax match mmaSymbol "\\\[FormalW\]"                         conceal cchar=w
    syntax match mmaSymbol "\\\[FormalX\]"                         conceal cchar=x
    syntax match mmaSymbol "\\\[FormalXi\]"                        conceal cchar=ξ
    syntax match mmaSymbol "\\\[FormalY\]"                         conceal cchar=y
    syntax match mmaSymbol "\\\[FormalZ\]"                         conceal cchar=z
    syntax match mmaSymbol "\\\[FormalZeta\]"                      conceal cchar=ζ
    syntax match mmaSymbol "\\\[FreakedSmiley\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[Function\]"                        conceal cchar=
    syntax match mmaSymbol "\\\[Gamma\]"                           conceal cchar=γ
    syntax match mmaSymbol "\\\[GeminiSign\]"                      conceal cchar=♊
    syntax match mmaSymbol "\\\[Gimel\]"                           conceal cchar=ℷ
    syntax match mmaSymbol "\\\[GothicA\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicB\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicC\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalA\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalB\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalC\]"                  conceal cchar=ℭ
    syntax match mmaSymbol "\\\[GothicCapitalD\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalE\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalF\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalG\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalH\]"                  conceal cchar=ℌ
    syntax match mmaSymbol "\\\[GothicCapitalI\]"                  conceal cchar=ℑ
    syntax match mmaSymbol "\\\[GothicCapitalJ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalK\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalL\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalM\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalN\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalO\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalP\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalQ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalR\]"                  conceal cchar=ℜ
    syntax match mmaSymbol "\\\[GothicCapitalS\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalT\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalU\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalV\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalW\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalX\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalY\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[GothicCapitalZ\]"                  conceal cchar=ℨ
    syntax match mmaSymbol "\\\[GothicD\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicE\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicEight\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[GothicF\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicFive\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[GothicFour\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[GothicG\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicH\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicI\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicJ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicK\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicL\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicM\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicN\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicNine\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[GothicO\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicOne\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[GothicP\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicQ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicR\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicS\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicSeven\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[GothicSix\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[GothicT\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicThree\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[GothicTwo\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[GothicU\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicV\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicW\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicX\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicY\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicZ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[GothicZero\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[GrayCircle\]"                      conceal cchar=●
    syntax match mmaSymbol "\\\[GraySquare\]"                      conceal cchar=■
    syntax match mmaSymbol "\\\[GreaterEqualLess\]"                conceal cchar=⋛
    syntax match mmaSymbol "\\\[GreaterEqual\]"                    conceal cchar=≥
    syntax match mmaSymbol "\\\[GreaterFullEqual\]"                conceal cchar=≧
    syntax match mmaSymbol "\\\[GreaterGreater\]"                  conceal cchar=≫
    syntax match mmaSymbol "\\\[GreaterLess\]"                     conceal cchar=≷
    syntax match mmaSymbol "\\\[GreaterSlantEqual\]"               conceal cchar=⩾
    syntax match mmaSymbol "\\\[GreaterTilde\]"                    conceal cchar=≳
    syntax match mmaSymbol "\\\[Hacek\]"                           conceal cchar=ˇ
    syntax match mmaSymbol "\\\[HappySmiley\]"                     conceal cchar=☺
    syntax match mmaSymbol "\\\[HBar\]"                            conceal cchar=ℏ
    syntax match mmaSymbol "\\\[HeartSuit\]"                       conceal cchar=♡
    syntax match mmaSymbol "\\\[HermitianConjugate\]"              conceal cchar=
    syntax match mmaSymbol "\\\[HorizontalLine\]"                  conceal cchar=─
    syntax match mmaSymbol "\\\[HumpDownHump\]"                    conceal cchar=≎
    syntax match mmaSymbol "\\\[HumpEqual\]"                       conceal cchar=≏
    syntax match mmaSymbol "\\\[Hyphen\]"                          conceal cchar=‐
    syntax match mmaSymbol "\\\[IAcute\]"                          conceal cchar=í
    syntax match mmaSymbol "\\\[ICup\]"                            conceal cchar=ĭ
    syntax match mmaSymbol "\\\[IDoubleDot\]"                      conceal cchar=ï
    syntax match mmaSymbol "\\\[IGrave\]"                          conceal cchar=ì
    syntax match mmaSymbol "\\\[IHat\]"                            conceal cchar=î
    syntax match mmaSymbol "\\\[ImaginaryI\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[ImaginaryJ\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[Implies\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[Infinity\]"                        conceal cchar=∞
    syntax match mmaSymbol "\\\[Integral\]"                        conceal cchar=∫
    syntax match mmaSymbol "\\\[Intersection\]"                    conceal cchar=⋂
    syntax match mmaSymbol "\\\[Iota\]"                            conceal cchar=ι
    syntax match mmaSymbol "\\\[Jupiter\]"                         conceal cchar=♃
    syntax match mmaSymbol "\\\[Kappa\]"                           conceal cchar=κ
    syntax match mmaSymbol "\\\[KernelIcon\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[Koppa\]"                           conceal cchar=ϟ
    syntax match mmaSymbol "\\\[Lambda\]"                          conceal cchar=λ
    syntax match mmaSymbol "\\\[LastPage\]"                        conceal cchar=
    syntax match mmaSymbol "\\\[LeftAngleBracket\]"                conceal cchar=〈
    syntax match mmaSymbol "\\\[LeftArrowBar\]"                    conceal cchar=⇤
    syntax match mmaSymbol "\\\[LeftArrow\]"                       conceal cchar=←
    syntax match mmaSymbol "\\\[LeftArrowRightArrow\]"             conceal cchar=⇆
    syntax match mmaSymbol "\\\[LeftAssociation\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[LeftBracketingBar\]"               conceal cchar=
    syntax match mmaSymbol "\\\[LeftCeiling\]"                     conceal cchar=⌈
    syntax match mmaSymbol "\\\[LeftDoubleBracket\]"               conceal cchar=〚
    syntax match mmaSymbol "\\\[LeftDoubleBracketingBar\]"         conceal cchar=
    syntax match mmaSymbol "\\\[LeftDownTeeVector\]"               conceal cchar=⥡
    syntax match mmaSymbol "\\\[LeftDownVectorBar\]"               conceal cchar=⥙
    syntax match mmaSymbol "\\\[LeftDownVector\]"                  conceal cchar=⇃
    syntax match mmaSymbol "\\\[LeftFloor\]"                       conceal cchar=⌊
    syntax match mmaSymbol "\\\[LeftGuillemet\]"                   conceal cchar=«
    syntax match mmaSymbol "\\\[LeftModified\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[LeftPointer\]"                     conceal cchar=◂
    syntax match mmaSymbol "\\\[LeftRightArrow\]"                  conceal cchar=↔
    syntax match mmaSymbol "\\\[LeftRightVector\]"                 conceal cchar=⥎
    syntax match mmaSymbol "\\\[LeftSkeleton\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[LeftTee\]"                         conceal cchar=⊣
    syntax match mmaSymbol "\\\[LeftTeeArrow\]"                    conceal cchar=↤
    syntax match mmaSymbol "\\\[LeftTeeVector\]"                   conceal cchar=⥚
    syntax match mmaSymbol "\\\[LeftTriangle\]"                    conceal cchar=⊲
    syntax match mmaSymbol "\\\[LeftTriangleBar\]"                 conceal cchar=⧏
    syntax match mmaSymbol "\\\[LeftTriangleEqual\]"               conceal cchar=⊴
    syntax match mmaSymbol "\\\[LeftUpDownVector\]"                conceal cchar=⥑
    syntax match mmaSymbol "\\\[LeftUpTeeVector\]"                 conceal cchar=⥠
    syntax match mmaSymbol "\\\[LeftUpVector\]"                    conceal cchar=↿
    syntax match mmaSymbol "\\\[LeftUpVectorBar\]"                 conceal cchar=⥘
    syntax match mmaSymbol "\\\[LeftVector\]"                      conceal cchar=↼
    syntax match mmaSymbol "\\\[LeftVectorBar\]"                   conceal cchar=⥒
    syntax match mmaSymbol "\\\[LeoSign\]"                         conceal cchar=♌
    syntax match mmaSymbol "\\\[LessEqual\]"                       conceal cchar=≤
    syntax match mmaSymbol "\\\[LessEqualGreater\]"                conceal cchar=⋚
    syntax match mmaSymbol "\\\[LessFullEqual\]"                   conceal cchar=≦
    syntax match mmaSymbol "\\\[LessGreater\]"                     conceal cchar=≶
    syntax match mmaSymbol "\\\[LessLess\]"                        conceal cchar=≪
    syntax match mmaSymbol "\\\[LessSlantEqual\]"                  conceal cchar=⩽
    syntax match mmaSymbol "\\\[LessTilde\]"                       conceal cchar=≲
    syntax match mmaSymbol "\\\[LibraSign\]"                       conceal cchar=♎
    syntax match mmaSymbol "\\\[LightBulb\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[LongDash\]"                        conceal cchar=—
    syntax match mmaSymbol "\\\[LongEqual\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[LongLeftArrow\]"                   conceal cchar=⟵
    syntax match mmaSymbol "\\\[LongLeftRightArrow\]"              conceal cchar=⟷
    syntax match mmaSymbol "\\\[LongRightArrow\]"                  conceal cchar=⟶
    syntax match mmaSymbol "\\\[LowerLeftArrow\]"                  conceal cchar=↙
    syntax match mmaSymbol "\\\[LowerRightArrow\]"                 conceal cchar=↘
    syntax match mmaSymbol "\\\[LSlash\]"                          conceal cchar=ł
    syntax match mmaSymbol "\\\[Mars\]"                            conceal cchar=♂
    syntax match mmaSymbol "\\\[MathematicaIcon\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[MeasuredAngle\]"                   conceal cchar=∡
    syntax match mmaSymbol "\\\[MinusPlus\]"                       conceal cchar=∓
    syntax match mmaSymbol "\\\[Mu\]"                              conceal cchar=μ
    syntax match mmaSymbol "\\\[Nand\]"                            conceal cchar=⊼
    syntax match mmaSymbol "\\\[Natural\]"                         conceal cchar=♮
    syntax match mmaSymbol "\\\[Neptune\]"                         conceal cchar=♆
    syntax match mmaSymbol "\\\[NestedGreaterGreater\]"            conceal cchar=⪢
    syntax match mmaSymbol "\\\[NestedLessLess\]"                  conceal cchar=⪡
    syntax match mmaSymbol "\\\[NeutralSmiley\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[NHacek\]"                          conceal cchar=ň
    syntax match mmaSymbol "\\\[Nor\]"                             conceal cchar=⊽
    syntax match mmaSymbol "\\\[NotCongruent\]"                    conceal cchar=≢
    syntax match mmaSymbol "\\\[NotCupCap\]"                       conceal cchar=≭
    syntax match mmaSymbol "\\\[NotDoubleVerticalBar\]"            conceal cchar=∦
    syntax match mmaSymbol "\\\[NotElement\]"                      conceal cchar=∉
    syntax match mmaSymbol "\\\[NotEqual\]"                        conceal cchar=≠
    syntax match mmaSymbol "\\\[NotEqualTilde\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[NotExists\]"                       conceal cchar=∄
    syntax match mmaSymbol "\\\[NotGreater\]"                      conceal cchar=≯
    syntax match mmaSymbol "\\\[NotGreaterEqual\]"                 conceal cchar=≱
    syntax match mmaSymbol "\\\[NotGreaterFullEqual\]"             conceal cchar=≩
    syntax match mmaSymbol "\\\[NotGreaterGreater\]"               conceal cchar=
    syntax match mmaSymbol "\\\[NotGreaterLess\]"                  conceal cchar=≹
    syntax match mmaSymbol "\\\[NotGreaterSlantEqual\]"            conceal cchar=
    syntax match mmaSymbol "\\\[NotGreaterTilde\]"                 conceal cchar=≵
    syntax match mmaSymbol "\\\[NotHumpDownHump\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[NotHumpEqual\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[NotLeftTriangle\]"                 conceal cchar=⋪
    syntax match mmaSymbol "\\\[NotLeftTriangleBar\]"              conceal cchar=
    syntax match mmaSymbol "\\\[NotLeftTriangleEqual\]"            conceal cchar=⋬
    syntax match mmaSymbol "\\\[NotLessEqual\]"                    conceal cchar=≰
    syntax match mmaSymbol "\\\[NotLessFullEqual\]"                conceal cchar=≨
    syntax match mmaSymbol "\\\[NotLessGreater\]"                  conceal cchar=≸
    syntax match mmaSymbol "\\\[NotLess\]"                         conceal cchar=≮
    syntax match mmaSymbol "\\\[NotLessLess\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[NotLessSlantEqual\]"               conceal cchar=
    syntax match mmaSymbol "\\\[NotLessTilde\]"                    conceal cchar=≴
    syntax match mmaSymbol "\\\[Not\]"                             conceal cchar=¬
    syntax match mmaSymbol "\\\[NotNestedGreaterGreater\]"         conceal cchar=
    syntax match mmaSymbol "\\\[NotNestedLessLess\]"               conceal cchar=
    syntax match mmaSymbol "\\\[NotPrecedes\]"                     conceal cchar=⊀
    syntax match mmaSymbol "\\\[NotPrecedesEqual\]"                conceal cchar=
    syntax match mmaSymbol "\\\[NotPrecedesSlantEqual\]"           conceal cchar=⋠
    syntax match mmaSymbol "\\\[NotPrecedesTilde\]"                conceal cchar=⋨
    syntax match mmaSymbol "\\\[NotReverseElement\]"               conceal cchar=∌
    syntax match mmaSymbol "\\\[NotRightTriangle\]"                conceal cchar=⋫
    syntax match mmaSymbol "\\\[NotRightTriangleBar\]"             conceal cchar=
    syntax match mmaSymbol "\\\[NotRightTriangleEqual\]"           conceal cchar=⋭
    syntax match mmaSymbol "\\\[NotSquareSubset\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[NotSquareSubsetEqual\]"            conceal cchar=⋢
    syntax match mmaSymbol "\\\[NotSquareSuperset\]"               conceal cchar=
    syntax match mmaSymbol "\\\[NotSquareSupersetEqual\]"          conceal cchar=⋣
    syntax match mmaSymbol "\\\[NotSubset\]"                       conceal cchar=⊄
    syntax match mmaSymbol "\\\[NotSubsetEqual\]"                  conceal cchar=⊈
    syntax match mmaSymbol "\\\[NotSucceeds\]"                     conceal cchar=⊁
    syntax match mmaSymbol "\\\[NotSucceedsEqual\]"                conceal cchar=
    syntax match mmaSymbol "\\\[NotSucceedsSlantEqual\]"           conceal cchar=⋡
    syntax match mmaSymbol "\\\[NotSucceedsTilde\]"                conceal cchar=⋩
    syntax match mmaSymbol "\\\[NotSuperset\]"                     conceal cchar=⊅
    syntax match mmaSymbol "\\\[NotSupersetEqual\]"                conceal cchar=⊉
    syntax match mmaSymbol "\\\[NotTilde\]"                        conceal cchar=≁
    syntax match mmaSymbol "\\\[NotTildeEqual\]"                   conceal cchar=≄
    syntax match mmaSymbol "\\\[NotTildeFullEqual\]"               conceal cchar=≇
    syntax match mmaSymbol "\\\[NotTildeTilde\]"                   conceal cchar=≉
    syntax match mmaSymbol "\\\[NotVerticalBar\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[NTilde\]"                          conceal cchar=ñ
    syntax match mmaSymbol "\\\[Nu\]"                              conceal cchar=ν
    syntax match mmaSymbol "\\\[NumberSign\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[OAcute\]"                          conceal cchar=ó
    syntax match mmaSymbol "\\\[ODoubleAcute\]"                    conceal cchar=ő
    syntax match mmaSymbol "\\\[ODoubleDot\]"                      conceal cchar=ö
    syntax match mmaSymbol "\\\[OE\]"                              conceal cchar=œ
    syntax match mmaSymbol "\\\[OGrave\]"                          conceal cchar=ò
    syntax match mmaSymbol "\\\[OHat\]"                            conceal cchar=ô
    syntax match mmaSymbol "\\\[Omega\]"                           conceal cchar=ω
    syntax match mmaSymbol "\\\[Omicron\]"                         conceal cchar=ο
    syntax match mmaSymbol "\\\[OpenCurlyDoubleQuote\]"            conceal cchar=“
    syntax match mmaSymbol "\\\[OpenCurlyQuote\]"                  conceal cchar=‘
    syntax match mmaSymbol "\\\[OptionKey\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[Or\]"                              conceal cchar=∨
    syntax match mmaSymbol "\\\[OSlash\]"                          conceal cchar=ø
    syntax match mmaSymbol "\\\[OTilde\]"                          conceal cchar=õ
    syntax match mmaSymbol "\\\[OverBrace\]"                       conceal cchar=︷
    syntax match mmaSymbol "\\\[OverBracket\]"                     conceal cchar=⎴
    syntax match mmaSymbol "\\\[OverParenthesis\]"                 conceal cchar=︵
    syntax match mmaSymbol "\\\[Paragraph\]"                       conceal cchar=¶
    syntax match mmaSymbol "\\\[PartialD\]"                        conceal cchar=∂
    syntax match mmaSymbol "\\\[PermutationProduct\]"              conceal cchar=
    syntax match mmaSymbol "\\\[Phi\]"                             conceal cchar=ϕ
    syntax match mmaSymbol "\\\[Pi\]"                              conceal cchar=π
    syntax match mmaSymbol "\\\[Piecewise\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[PiscesSign\]"                      conceal cchar=♓
    syntax match mmaSymbol "\\\[Placeholder\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[PlusMinus\]"                       conceal cchar=±
    syntax match mmaSymbol "\\\[Pluto\]"                           conceal cchar=♇
    syntax match mmaSymbol "\\\[Precedes\]"                        conceal cchar=≺
    syntax match mmaSymbol "\\\[PrecedesEqual\]"                   conceal cchar=⪯
    syntax match mmaSymbol "\\\[PrecedesSlantEqual\]"              conceal cchar=≼
    syntax match mmaSymbol "\\\[PrecedesTilde\]"                   conceal cchar=≾
    syntax match mmaSymbol "\\\[Prime\]"                           conceal cchar=′
    syntax match mmaSymbol "\\\[Product\]"                         conceal cchar=∏
    syntax match mmaSymbol "\\\[Proportion\]"                      conceal cchar=∷
    syntax match mmaSymbol "\\\[Proportional\]"                    conceal cchar=∝
    syntax match mmaSymbol "\\\[Psi\]"                             conceal cchar=ψ
    syntax match mmaSymbol "\\\[QuarterNote\]"                     conceal cchar=♩
    syntax match mmaSymbol "\\\[RegisteredTrademark\]"             conceal cchar=®
    syntax match mmaSymbol "\\\[ReturnIndicator\]"                 conceal cchar=↵
    syntax match mmaSymbol "\\\[ReturnKey\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[ReverseDoublePrime\]"              conceal cchar=‶
    syntax match mmaSymbol "\\\[ReverseElement\]"                  conceal cchar=∋
    syntax match mmaSymbol "\\\[ReverseEquilibrium\]"              conceal cchar=⇋
    syntax match mmaSymbol "\\\[ReversePrime\]"                    conceal cchar=‵
    syntax match mmaSymbol "\\\[ReverseUpEquilibrium\]"            conceal cchar=⥯
    syntax match mmaSymbol "\\\[RHacek\]"                          conceal cchar=ř
    syntax match mmaSymbol "\\\[Rho\]"                             conceal cchar=ρ
    syntax match mmaSymbol "\\\[RightAngle\]"                      conceal cchar=∟
    syntax match mmaSymbol "\\\[RightAngleBracket\]"               conceal cchar=〉
    syntax match mmaSymbol "\\\[RightArrow\]"                      conceal cchar=→
    syntax match mmaSymbol "\\\[RightArrowBar\]"                   conceal cchar=⇥
    syntax match mmaSymbol "\\\[RightArrowLeftArrow\]"             conceal cchar=⇄
    syntax match mmaSymbol "\\\[RightAssociation\]"                conceal cchar=
    syntax match mmaSymbol "\\\[RightBracketingBar\]"              conceal cchar=
    syntax match mmaSymbol "\\\[RightCeiling\]"                    conceal cchar=⌉
    syntax match mmaSymbol "\\\[RightDoubleBracket\]"              conceal cchar=〛
    syntax match mmaSymbol "\\\[RightDoubleBracketingBar\]"        conceal cchar=
    syntax match mmaSymbol "\\\[RightDownTeeVector\]"              conceal cchar=⥝
    syntax match mmaSymbol "\\\[RightDownVector\]"                 conceal cchar=⇂
    syntax match mmaSymbol "\\\[RightDownVectorBar\]"              conceal cchar=⥕
    syntax match mmaSymbol "\\\[RightFloor\]"                      conceal cchar=⌋
    syntax match mmaSymbol "\\\[RightGuillemet\]"                  conceal cchar=»
    syntax match mmaSymbol "\\\[RightModified\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[RightPointer\]"                    conceal cchar=▸
    syntax match mmaSymbol "\\\[RightSkeleton\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[RightTee\]"                        conceal cchar=⊢
    syntax match mmaSymbol "\\\[RightTeeArrow\]"                   conceal cchar=↦
    syntax match mmaSymbol "\\\[RightTeeVector\]"                  conceal cchar=⥛
    syntax match mmaSymbol "\\\[RightTriangle\]"                   conceal cchar=⊳
    syntax match mmaSymbol "\\\[RightTriangleBar\]"                conceal cchar=⧐
    syntax match mmaSymbol "\\\[RightTriangleEqual\]"              conceal cchar=⊵
    syntax match mmaSymbol "\\\[RightUpDownVector\]"               conceal cchar=⥏
    syntax match mmaSymbol "\\\[RightUpTeeVector\]"                conceal cchar=⥜
    syntax match mmaSymbol "\\\[RightUpVector\]"                   conceal cchar=↾
    syntax match mmaSymbol "\\\[RightUpVectorBar\]"                conceal cchar=⥔
    syntax match mmaSymbol "\\\[RightVector\]"                     conceal cchar=⇀
    syntax match mmaSymbol "\\\[RightVectorBar\]"                  conceal cchar=⥓
    syntax match mmaSymbol "\\\[RoundImplies\]"                    conceal cchar=⥰
    syntax match mmaSymbol "\\\[RoundSpaceIndicator\]"             conceal cchar=
    syntax match mmaSymbol "\\\[Rule\]"                            conceal cchar=
    syntax match mmaSymbol "\\\[RuleDelayed\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[SadSmiley\]"                       conceal cchar=☹
    syntax match mmaSymbol "\\\[SagittariusSign\]"                 conceal cchar=♐
    syntax match mmaSymbol "\\\[Sampi\]"                           conceal cchar=ϡ
    syntax match mmaSymbol "\\\[Saturn\]"                          conceal cchar=♄
    syntax match mmaSymbol "\\\[ScorpioSign\]"                     conceal cchar=♏
    syntax match mmaSymbol "\\\[ScriptA\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptB\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptC\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalA\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalB\]"                  conceal cchar=ℬ
    syntax match mmaSymbol "\\\[ScriptCapitalC\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalD\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalE\]"                  conceal cchar=ℰ
    syntax match mmaSymbol "\\\[ScriptCapitalF\]"                  conceal cchar=ℱ
    syntax match mmaSymbol "\\\[ScriptCapitalG\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalH\]"                  conceal cchar=ℋ
    syntax match mmaSymbol "\\\[ScriptCapitalI\]"                  conceal cchar=ℐ
    syntax match mmaSymbol "\\\[ScriptCapitalJ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalK\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalL\]"                  conceal cchar=ℒ
    syntax match mmaSymbol "\\\[ScriptCapitalM\]"                  conceal cchar=ℳ
    syntax match mmaSymbol "\\\[ScriptCapitalN\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalO\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalP\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalQ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalR\]"                  conceal cchar=ℛ
    syntax match mmaSymbol "\\\[ScriptCapitalS\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalT\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalU\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalV\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalW\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalX\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalY\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptCapitalZ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptD\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptDotlessI\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptDotlessJ\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ScriptE\]"                         conceal cchar=ℯ
    syntax match mmaSymbol "\\\[ScriptEight\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[ScriptF\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptFive\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[ScriptFour\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[ScriptG\]"                         conceal cchar=ℊ
    syntax match mmaSymbol "\\\[ScriptH\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptI\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptJ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptK\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptL\]"                         conceal cchar=ℓ
    syntax match mmaSymbol "\\\[ScriptM\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptN\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptNine\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[ScriptO\]"                         conceal cchar=ℴ
    syntax match mmaSymbol "\\\[ScriptOne\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[ScriptP\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptQ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptR\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptS\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptSeven\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[ScriptSix\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[ScriptT\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptThree\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[ScriptTwo\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[ScriptU\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptV\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptW\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptX\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptY\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptZ\]"                         conceal cchar=
    syntax match mmaSymbol "\\\[ScriptZero\]"                      conceal cchar=
    syntax match mmaSymbol "\\\[Section\]"                         conceal cchar=§
    syntax match mmaSymbol "\\\[SelectionPlaceholder\]"            conceal cchar=
    syntax match mmaSymbol "\\\[SHacek\]"                          conceal cchar=š
    syntax match mmaSymbol "\\\[Sharp\]"                           conceal cchar=♯
    syntax match mmaSymbol "\\\[ShortDownArrow\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ShortLeftArrow\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[ShortRightArrow\]"                 conceal cchar=
    syntax match mmaSymbol "\\\[ShortUpArrow\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[Sigma\]"                           conceal cchar=σ
    syntax match mmaSymbol "\\\[SixPointedStar\]"                  conceal cchar=✶
    syntax match mmaSymbol "\\\[SkeletonIndicator\]"               conceal cchar=⁃
    syntax match mmaSymbol "\\\[SmallCircle\]"                     conceal cchar=∘
    syntax match mmaSymbol "\\\[SpaceIndicator\]"                  conceal cchar=␣
    syntax match mmaSymbol "\\\[SpaceKey\]"                        conceal cchar=
    syntax match mmaSymbol "\\\[SpadeSuit\]"                       conceal cchar=♠
    syntax match mmaSymbol "\\\[SpanFromAbove\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[SpanFromBoth\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[SpanFromLeft\]"                    conceal cchar=
    syntax match mmaSymbol "\\\[SphericalAngle\]"                  conceal cchar=∢
    syntax match mmaSymbol "\\\[Sqrt\]"                            conceal cchar=√
    syntax match mmaSymbol "\\\[Square\]"                          conceal cchar=
    syntax match mmaSymbol "\\\[SquareIntersection\]"              conceal cchar=⊓
    syntax match mmaSymbol "\\\[SquareSubset\]"                    conceal cchar=⊏
    syntax match mmaSymbol "\\\[SquareSubsetEqual\]"               conceal cchar=⊑
    syntax match mmaSymbol "\\\[SquareSuperset\]"                  conceal cchar=⊐
    syntax match mmaSymbol "\\\[SquareSupersetEqual\]"             conceal cchar=⊒
    syntax match mmaSymbol "\\\[SquareUnion\]"                     conceal cchar=⊔
    syntax match mmaSymbol "\\\[Star\]"                            conceal cchar=⋆
    syntax match mmaSymbol "\\\[Sterling\]"                        conceal cchar=£
    syntax match mmaSymbol "\\\[Stigma\]"                          conceal cchar=ϛ
    syntax match mmaSymbol "\\\[Subset\]"                          conceal cchar=⊂
    syntax match mmaSymbol "\\\[SubsetEqual\]"                     conceal cchar=⊆
    syntax match mmaSymbol "\\\[Succeeds\]"                        conceal cchar=≻
    syntax match mmaSymbol "\\\[SucceedsEqual\]"                   conceal cchar=⪰
    syntax match mmaSymbol "\\\[SucceedsSlantEqual\]"              conceal cchar=≽
    syntax match mmaSymbol "\\\[SucceedsTilde\]"                   conceal cchar=≿
    syntax match mmaSymbol "\\\[SuchThat\]"                        conceal cchar=∍
    syntax match mmaSymbol "\\\[Sum\]"                             conceal cchar=∑
    syntax match mmaSymbol "\\\[Superset\]"                        conceal cchar=⊃
    syntax match mmaSymbol "\\\[SupersetEqual\]"                   conceal cchar=⊇
    syntax match mmaSymbol "\\\[SystemEnterKey\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[SystemsModelDelay\]"               conceal cchar=
    syntax match mmaSymbol "\\\[SZ\]"                              conceal cchar=ß
    syntax match mmaSymbol "\\\[TabKey\]"                          conceal cchar=
    syntax match mmaSymbol "\\\[Tau\]"                             conceal cchar=τ
    syntax match mmaSymbol "\\\[TaurusSign\]"                      conceal cchar=♉
    syntax match mmaSymbol "\\\[TensorProduct\]"                   conceal cchar=
    syntax match mmaSymbol "\\\[TensorWedge\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[THacek\]"                          conceal cchar=ť
    syntax match mmaSymbol "\\\[Therefore\]"                       conceal cchar=∴
    syntax match mmaSymbol "\\\[Theta\]"                           conceal cchar=θ
    syntax match mmaSymbol "\\\[Thorn\]"                           conceal cchar=þ
    syntax match mmaSymbol "\\\[Tilde\]"                           conceal cchar=∼
    syntax match mmaSymbol "\\\[TildeEqual\]"                      conceal cchar=≃
    syntax match mmaSymbol "\\\[TildeFullEqual\]"                  conceal cchar=≅
    syntax match mmaSymbol "\\\[TildeTilde\]"                      conceal cchar=≈
    syntax match mmaSymbol "\\\[Times\]"                           conceal cchar=×
    syntax match mmaSymbol "\\\[Trademark\]"                       conceal cchar=™
    syntax match mmaSymbol "\\\[Transpose\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[TripleDot\]"                       conceal cchar=
    syntax match mmaSymbol "\\\[UAcute\]"                          conceal cchar=ú
    syntax match mmaSymbol "\\\[UDoubleAcute\]"                    conceal cchar=ű
    syntax match mmaSymbol "\\\[UDoubleDot\]"                      conceal cchar=ü
    syntax match mmaSymbol "\\\[UGrave\]"                          conceal cchar=ù
    syntax match mmaSymbol "\\\[UHat\]"                            conceal cchar=û
    syntax match mmaSymbol "\\\[UnderBrace\]"                      conceal cchar=︸
    syntax match mmaSymbol "\\\[UnderBracket\]"                    conceal cchar=⎵
    syntax match mmaSymbol "\\\[UnderParenthesis\]"                conceal cchar=︶
    syntax match mmaSymbol "\\\[UndirectedEdge\]"                  conceal cchar=
    syntax match mmaSymbol "\\\[Union\]"                           conceal cchar=⋃
    syntax match mmaSymbol "\\\[UnionPlus\]"                       conceal cchar=⊎
    syntax match mmaSymbol "\\\[UpArrow\]"                         conceal cchar=↑
    syntax match mmaSymbol "\\\[UpArrowBar\]"                      conceal cchar=⤒
    syntax match mmaSymbol "\\\[UpArrowDownArrow\]"                conceal cchar=⇅
    syntax match mmaSymbol "\\\[UpDownArrow\]"                     conceal cchar=↕
    syntax match mmaSymbol "\\\[UpEquilibrium\]"                   conceal cchar=⥮
    syntax match mmaSymbol "\\\[UpperLeftArrow\]"                  conceal cchar=↖
    syntax match mmaSymbol "\\\[UpperRightArrow\]"                 conceal cchar=↗
    syntax match mmaSymbol "\\\[UpPointer\]"                       conceal cchar=▴
    syntax match mmaSymbol "\\\[Upsilon\]"                         conceal cchar=υ
    syntax match mmaSymbol "\\\[UpTee\]"                           conceal cchar=⊥
    syntax match mmaSymbol "\\\[UpTeeArrow\]"                      conceal cchar=↥
    syntax match mmaSymbol "\\\[Uranus\]"                          conceal cchar=⛢
    syntax match mmaSymbol "\\\[URing\]"                           conceal cchar=ů
    syntax match mmaSymbol "\\\[Vee\]"                             conceal cchar=⋁
    syntax match mmaSymbol "\\\[Venus\]"                           conceal cchar=♀
    syntax match mmaSymbol "\\\[VerticalBar\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[VerticalEllipsis\]"                conceal cchar=⋮
    syntax match mmaSymbol "\\\[VerticalLine\]"                    conceal cchar=│
    syntax match mmaSymbol "\\\[VerticalSeparator\]"               conceal cchar=
    syntax match mmaSymbol "\\\[VerticalTilde\]"                   conceal cchar=≀
    syntax match mmaSymbol "\\\[VirgoSign\]"                       conceal cchar=♍
    syntax match mmaSymbol "\\\[WarningSign\]"                     conceal cchar=
    syntax match mmaSymbol "\\\[WatchIcon\]"                       conceal cchar=⌚
    syntax match mmaSymbol "\\\[Wedge\]"                           conceal cchar=⋀
    syntax match mmaSymbol "\\\[WeierstrassP\]"                    conceal cchar=℘
    syntax match mmaSymbol "\\\[WhiteBishop\]"                     conceal cchar=♗
    syntax match mmaSymbol "\\\[WhiteKing\]"                       conceal cchar=♔
    syntax match mmaSymbol "\\\[WhiteKnight\]"                     conceal cchar=♘
    syntax match mmaSymbol "\\\[WhitePawn\]"                       conceal cchar=♙
    syntax match mmaSymbol "\\\[WhiteQueen\]"                      conceal cchar=♕
    syntax match mmaSymbol "\\\[WhiteRook\]"                       conceal cchar=♖
    syntax match mmaSymbol "\\\[Wolf\]"                            conceal cchar=
    syntax match mmaSymbol "\\\[WolframLanguageLogo\]"             conceal cchar=
    syntax match mmaSymbol "\\\[WolframLanguageLogoCircle\]"       conceal cchar=
    syntax match mmaSymbol "\\\[Xi\]"                              conceal cchar=ξ
    syntax match mmaSymbol "\\\[Xnor\]"                            conceal cchar=
    syntax match mmaSymbol "\\\[Xor\]"                             conceal cchar=⊻
    syntax match mmaSymbol "\\\[YAcute\]"                          conceal cchar=ý
    syntax match mmaSymbol "\\\[YDoubleDot\]"                      conceal cchar=ÿ
    syntax match mmaSymbol "\\\[Yen\]"                             conceal cchar=¥
    syntax match mmaSymbol "\\\[Zeta\]"                            conceal cchar=ζ
    syntax match mmaSymbol "\\\[ZHacek\]"                          conceal cchar=ž
endif

"These might be troublesome if the appropriate fonts are missing. Also, they don't
"look quite as good as the earlier ones, so enable only if the user chooses to
if exists('g:mma_candy') && g:mma_candy == 2
	"Constants
	syntax keyword mmaSystemSymbol I       conceal cchar=ⅈ
	syntax keyword mmaSystemSymbol E       conceal cchar=ⅇ

	"Functions
	syntax keyword mmaSystemSymbol Sum     conceal cchar=∑
	syntax keyword mmaSystemSymbol Product conceal cchar=∏
	syntax keyword mmaSystemSymbol Sqrt    conceal cchar=√

	"Misc
	syntax match mmaOperator ">>"          conceal cchar=» "Put
	syntax match mmaOperator "<<"          conceal cchar=« "Get
endif

hi! link Conceal Normal
setlocal conceallevel=2

