if polyglot#init#is_disabled(expand('<sfile>:p'), 'html5', 'syntax/html/rdfa.vim')
  finish
endif

" Vim syntax file
" Language:     RDFa
" Maintainer:   othree <othree@gmail.com>
" URL:          http://github.com/othree/html5-syntax.vim
" Last Change:  2012-06-08
" License:      MIT
" Changes:      update to Rec 07 June 2012

" RDFa
" http://www.w3.org/TR/rdfa-syntax/#s_syntax
syn keyword htmlArg contained about content datatype href inlist prefix property rel resource rev src typeof vocab
