if polyglot#init#is_disabled(expand('<sfile>:p'), 'llvm', 'ftplugin/mlir.vim')
  finish
endif

" Vim filetype plugin file
" Language: MLIR Assembly
" Maintainer: The MLIR team

if exists("b:did_ftplugin")
  finish
endif
let b:did_ftplugin = 1

setlocal softtabstop=2 shiftwidth=2
setlocal expandtab
setlocal comments+=://
