if polyglot#init#is_disabled(expand('<sfile>:p'), 'typescript', 'syntax/basic/decorator.vim')
  finish
endif

syntax match typescriptDecorator /@\([_$a-zA-Z][_$a-zA-Z0-9]*\.\)*[_$a-zA-Z][_$a-zA-Z0-9]*\>/
  \ nextgroup=typescriptFuncCallArg,typescriptTypeArguments
  \ contains=@_semantic,typescriptDotNotation
