if polyglot#init#is_disabled(expand('<sfile>:p'), 'aap', 'indent/aap.vim')
  finish
endif

" Vim indent file
" Language:	Aap recipe
" Maintainer:	Bram Moolenaar <Bram@vim.org>
" Last Change:	2005 Jun 24

" Only load this indent file when no other was loaded.
if exists("b:did_indent")
  finish
endif

" Works mostly like Python.
runtime! indent/python.vim
