if polyglot#init#is_disabled(expand('<sfile>:p'), 'typescript', 'syntax/yats/css.vim')
  finish
endif

syntax keyword typescriptDOMStyle contained alignContent alignItems alignSelf animation
syntax keyword typescriptDOMStyle contained animationDelay animationDirection animationDuration
syntax keyword typescriptDOMStyle contained animationFillMode animationIterationCount
syntax keyword typescriptDOMStyle contained animationName animationPlayState animationTimingFunction
syntax keyword typescriptDOMStyle contained appearance backfaceVisibility background
syntax keyword typescriptDOMStyle contained backgroundAttachment backgroundBlendMode
syntax keyword typescriptDOMStyle contained backgroundClip backgroundColor backgroundImage
syntax keyword typescriptDOMStyle contained backgroundOrigin backgroundPosition backgroundRepeat
syntax keyword typescriptDOMStyle contained backgroundSize border borderBottom borderBottomColor
syntax keyword typescriptDOMStyle contained borderBottomLeftRadius borderBottomRightRadius
syntax keyword typescriptDOMStyle contained borderBottomStyle borderBottomWidth borderCollapse
syntax keyword typescriptDOMStyle contained borderColor borderImage borderImageOutset
syntax keyword typescriptDOMStyle contained borderImageRepeat borderImageSlice borderImageSource
syntax keyword typescriptDOMStyle contained borderImageWidth borderLeft borderLeftColor
syntax keyword typescriptDOMStyle contained borderLeftStyle borderLeftWidth borderRadius
syntax keyword typescriptDOMStyle contained borderRight borderRightColor borderRightStyle
syntax keyword typescriptDOMStyle contained borderRightWidth borderSpacing borderStyle
syntax keyword typescriptDOMStyle contained borderTop borderTopColor borderTopLeftRadius
syntax keyword typescriptDOMStyle contained borderTopRightRadius borderTopStyle borderTopWidth
syntax keyword typescriptDOMStyle contained borderWidth bottom boxDecorationBreak
syntax keyword typescriptDOMStyle contained boxShadow boxSizing breakAfter breakBefore
syntax keyword typescriptDOMStyle contained breakInside captionSide caretColor caretShape
syntax keyword typescriptDOMStyle contained caret clear clip clipPath color columns
syntax keyword typescriptDOMStyle contained columnCount columnFill columnGap columnRule
syntax keyword typescriptDOMStyle contained columnRuleColor columnRuleStyle columnRuleWidth
syntax keyword typescriptDOMStyle contained columnSpan columnWidth content counterIncrement
syntax keyword typescriptDOMStyle contained counterReset cursor direction display
syntax keyword typescriptDOMStyle contained emptyCells flex flexBasis flexDirection
syntax keyword typescriptDOMStyle contained flexFlow flexGrow flexShrink flexWrap
syntax keyword typescriptDOMStyle contained float font fontFamily fontFeatureSettings
syntax keyword typescriptDOMStyle contained fontKerning fontLanguageOverride fontSize
syntax keyword typescriptDOMStyle contained fontSizeAdjust fontStretch fontStyle fontSynthesis
syntax keyword typescriptDOMStyle contained fontVariant fontVariantAlternates fontVariantCaps
syntax keyword typescriptDOMStyle contained fontVariantEastAsian fontVariantLigatures
syntax keyword typescriptDOMStyle contained fontVariantNumeric fontVariantPosition
syntax keyword typescriptDOMStyle contained fontWeight grad grid gridArea gridAutoColumns
syntax keyword typescriptDOMStyle contained gridAutoFlow gridAutoPosition gridAutoRows
syntax keyword typescriptDOMStyle contained gridColumn gridColumnStart gridColumnEnd
syntax keyword typescriptDOMStyle contained gridRow gridRowStart gridRowEnd gridTemplate
syntax keyword typescriptDOMStyle contained gridTemplateAreas gridTemplateRows gridTemplateColumns
syntax keyword typescriptDOMStyle contained height hyphens imageRendering imageResolution
syntax keyword typescriptDOMStyle contained imageOrientation imeMode inherit justifyContent
syntax keyword typescriptDOMStyle contained left letterSpacing lineBreak lineHeight
syntax keyword typescriptDOMStyle contained listStyle listStyleImage listStylePosition
syntax keyword typescriptDOMStyle contained listStyleType margin marginBottom marginLeft
syntax keyword typescriptDOMStyle contained marginRight marginTop marks mask maskType
syntax keyword typescriptDOMStyle contained maxHeight maxWidth minHeight minWidth
syntax keyword typescriptDOMStyle contained mixBlendMode objectFit objectPosition
syntax keyword typescriptDOMStyle contained opacity order orphans outline outlineColor
syntax keyword typescriptDOMStyle contained outlineOffset outlineStyle outlineWidth
syntax keyword typescriptDOMStyle contained overflow overflowWrap overflowX overflowY
syntax keyword typescriptDOMStyle contained overflowClipBox padding paddingBottom
syntax keyword typescriptDOMStyle contained paddingLeft paddingRight paddingTop pageBreakAfter
syntax keyword typescriptDOMStyle contained pageBreakBefore pageBreakInside perspective
syntax keyword typescriptDOMStyle contained perspectiveOrigin pointerEvents position
syntax keyword typescriptDOMStyle contained quotes resize right shapeImageThreshold
syntax keyword typescriptDOMStyle contained shapeMargin shapeOutside tableLayout tabSize
syntax keyword typescriptDOMStyle contained textAlign textAlignLast textCombineHorizontal
syntax keyword typescriptDOMStyle contained textDecoration textDecorationColor textDecorationLine
syntax keyword typescriptDOMStyle contained textDecorationStyle textIndent textOrientation
syntax keyword typescriptDOMStyle contained textOverflow textRendering textShadow
syntax keyword typescriptDOMStyle contained textTransform textUnderlinePosition top
syntax keyword typescriptDOMStyle contained touchAction transform transformOrigin
syntax keyword typescriptDOMStyle contained transformStyle transition transitionDelay
syntax keyword typescriptDOMStyle contained transitionDuration transitionProperty
syntax keyword typescriptDOMStyle contained transitionTimingFunction unicodeBidi unicodeRange
syntax keyword typescriptDOMStyle contained userSelect userZoom verticalAlign visibility
syntax keyword typescriptDOMStyle contained whiteSpace width willChange wordBreak
syntax keyword typescriptDOMStyle contained wordSpacing wordWrap writingMode zIndex
if exists("did_typescript_hilink") | HiLink typescriptDOMStyle Keyword
endif
