if polyglot#init#is_disabled(expand('<sfile>:p'), 'emblem', 'indent/emblem.vim')
  finish
endif

" Language:    emblem
" Maintainer:  Yulij Andreevich Lesov <yalesov@gmail.com>
" URL:         http://github.com/yalesov/vim-emblem
" Version:     2.0.1
" Last Change: 2016 Jul 6
" License:     ISC
